#!/usr/bin/env python
  
#Lists
categoryList = []
factList = []
caseList = []
saList = []
ruleList = []
erList = []
foList = []
evalList = []

#Operations  
AND = " AND "
OR = " OR "
NEG = " NEG "
IMPLY = " IMPLY "
EQUALS = " EQUALS "
LESSER = " LESSER "
GREATER = " GREATER "
  
#Classes
class Category(object):
    def __init__(self, name=None, description=None):
        self.name = name
        self.description = description

class Fact(object):
    def __init__(self, name=None, category=None, description=None, value=None):
        self.name = name
        self.category = category
        self.description = description
        self.value = value
          
class SecurityAttribute(object):
    def __init__(self, name=None, description=None):
        self.name = name
        self.description = description
          
class Rule(object): 
    def __init__(self, elements=[]):
        self.elements = elements #facts + operators
        
          
class EvaluationRule(object):
    def __init__(self, elements=None, influence=None, security_attribute=None):
        self.elements = elements #facts + operators
        self.influence = influence #influence value
        self.security_attribute = security_attribute
          
class FactsOrder(object):
    def __init__(self, elements=[], security_attribute=None):
        self.elements = elements
        self.security_attribute = security_attribute
        
class Case(object):
    def __init__(self, casename=None, facts=None, description=None):
        self.casename = casename
        self.facts = facts
        self.description = description
        
class Evaluation(object):
    def __init__(self, elements=[], description=None, results_name=[], results_value=[]):
        self.elements = elements
        self.description = description
        self.results = results #dictionary - sec_att: value


#!/usr/bin/env python

import wx
import os
from aqopa.gui.general_purpose_frame_gui import GeneralFrame

"""
@file       gui.py
@brief      gui file for the greenanalysis module
@author     Katarzyna Mazur
"""


class SingleVersionPanel(wx.Panel):
    def __init__(self, module, *args, **kwargs):
        wx.Panel.__init__(self, *args, **kwargs)

        self.module = module
        self.versionSimulator = {}

        # ################
        # VERSION BOX
        #################

        versionBox = wx.StaticBox(self, label="Version")
        versionsLabel = wx.StaticText(self, label="Choose Version To See\nAnalysis Results:")
        self.versionsList = wx.ComboBox(self, style=wx.TE_READONLY, size=(200, -1))
        self.versionsList.Bind(wx.EVT_COMBOBOX, self.OnVersionChanged)
        versionBoxSizer = wx.StaticBoxSizer(versionBox, wx.HORIZONTAL)
        versionBoxSizer.Add(versionsLabel, 0, wx.ALL | wx.ALIGN_CENTER, 5)
        versionBoxSizer.Add(wx.StaticText(self), 1, wx.ALL | wx.EXPAND, 5)
        versionBoxSizer.Add(self.versionsList, 1, wx.ALL | wx.ALIGN_CENTER, 5)

        ##################################
        # CO2 RESULTS BOX
        ##################################

        self.co2Box = wx.StaticBox(self, label="The Carbon Dioxide Emissions Analysis Results")
        self.co2Label = wx.StaticText(self, label="CO2 produced per   \none kWh [pounds]:")
        self.co2Input = wx.TextCtrl(self, size=(200, -1))
        co2Sizer = wx.BoxSizer(wx.HORIZONTAL)
        co2Sizer.Add(self.co2Label, 0, wx.ALL | wx.EXPAND, 5)
        co2Sizer.Add(wx.StaticText(self), 1, wx.ALL | wx.EXPAND, 5)
        co2Sizer.Add(self.co2Input, 1, wx.ALL | wx.EXPAND | wx.ALIGN_RIGHT, 5)
        hostsBox, hostsBoxSizer = self._BuildHostsBoxAndSizer()
        co2BoxSizer = wx.StaticBoxSizer(self.co2Box, wx.VERTICAL)
        co2BoxSizer.Add(co2Sizer, 0, wx.ALL | wx.EXPAND)
        co2BoxSizer.Add(wx.StaticText(self), 0, wx.ALL | wx.EXPAND, 5)
        co2BoxSizer.Add(hostsBoxSizer, 1, wx.ALL | wx.EXPAND)

        #################
        # BUTTONS LAY
        #################
        self.showCo2ResultsBtn = wx.Button(self, label="Show")
        self.showCo2ResultsBtn.Bind(wx.EVT_BUTTON, self.OnShowCo2ResultsBtnClicked)
        buttonsSizer = wx.BoxSizer(wx.HORIZONTAL)
        buttonsSizer.Add(wx.StaticText(self), 1, wx.ALL | wx.EXPAND, 5)
        buttonsSizer.Add(self.showCo2ResultsBtn, 0, wx.ALL | wx.EXPAND, 5)

        #################
        # MAIN LAY
        #################
        sizer = wx.BoxSizer(wx.VERTICAL)
        sizer.Add(versionBoxSizer, 0, wx.ALL | wx.EXPAND, 5)
        sizer.Add(co2BoxSizer, 0, wx.ALL | wx.EXPAND, 5)
        sizer.Add(buttonsSizer, 0, wx.ALL | wx.EXPAND, 5)
        self.SetSizer(sizer)

        self.SetVersionsResultsVisibility(False)

    def OnShowCo2ResultsBtnClicked(self, event):
        co2Text = self.co2Input.GetValue().strip()
        try:
            co2 = float(co2Text)
        except ValueError:
            wx.MessageBox("'%s' is not valid CO2 amount. Please correct it." % co2Text,
                          'Error', wx.OK | wx.ICON_ERROR)
            return

        versionName = self.versionsList.GetValue()
        simulator = self.versionSimulator[versionName]
        selected_host = self._GetSelectedHost(simulator)
        all_emissions = self.module.calculate_all_emissions(simulator, simulator.context.hosts, co2)

        # get some financial info from module
        minemission, minhost = self.module.get_min_emission(simulator, simulator.context.hosts)
        maxemission, maxhost = self.module.get_max_emission(simulator, simulator.context.hosts)
        total_emission = self.module.get_total_emission(simulator, simulator.context.hosts)
        avg_emission = self.module.get_avg_emission(simulator, simulator.context.hosts)
        curr_emission = all_emissions[selected_host]

        # after all calculations, build the GUI
        title = "Carbon Dioxide Emissions Analysis for Host: "
        title += selected_host.original_name()

        co2Window = GeneralFrame(self, "Carbon Dioxide Emissions Analysis Results", title, "modules_results.png")
        co2Panel = wx.Panel(co2Window)

        # conversion constant used to convert pounds to kgs
        conv_constant = 0.45539237

        # ########################################################################
        # ACTUAL CARBON DIOXIDE EMISSIONS
        #########################################################################
        actualEmissionsBox = wx.StaticBox(co2Panel, label="Actual Carbon Dioxide Emissions of CPU")
        actualEmissionsBoxSizer = wx.StaticBoxSizer(actualEmissionsBox, wx.VERTICAL)
        #########################################################################
        # emission of the selected host
        #########################################################################
        infoLabel = "Emission for Host: "
        hostInfoLabel = wx.StaticText(co2Panel, label=infoLabel)
        co2Label ="%.15f" % curr_emission + " pounds"
        hostCO2Label = wx.StaticText(co2Panel, label=co2Label)
        sizer1 = wx.BoxSizer(wx.HORIZONTAL)
        sizer1.Add(hostInfoLabel, 0, wx.ALL | wx.EXPAND, 5)
        sizer1.Add(wx.StaticText(self), 1, wx.ALL | wx.EXPAND, 5)
        sizer1.Add(hostCO2Label, 0, wx.ALL | wx.EXPAND, 5)
        #########################################################################
        # minimal emission of version (minimal emissions of every host in given version)
        #########################################################################
        infoLabel = "Minimal Version Emission (Host: " + minhost.original_name() + ")"
        hostInfoLabel = wx.StaticText(co2Panel, label=infoLabel)
        co2Label ="%.15f" % minemission + " pounds"
        hostCO2Label = wx.StaticText(co2Panel, label=co2Label)
        sizer2 = wx.BoxSizer(wx.HORIZONTAL)
        sizer2.Add(hostInfoLabel, 0, wx.ALL | wx.EXPAND, 5)
        sizer2.Add(wx.StaticText(self), 1, wx.ALL | wx.EXPAND, 5)
        sizer2.Add(hostCO2Label, 0, wx.ALL | wx.EXPAND, 5)
        #########################################################################
        # maximal emission of version (maximal emissions of every host in given version)
        #########################################################################
        infoLabel = "Maximal Version Emission (Host: " + maxhost.original_name() + ")"
        hostInfoLabel = wx.StaticText(co2Panel, label=infoLabel)
        co2Label ="%.15f" % maxemission + " pounds"
        hostCO2Label = wx.StaticText(co2Panel, label=co2Label)
        sizer3 = wx.BoxSizer(wx.HORIZONTAL)
        sizer3.Add(hostInfoLabel, 0, wx.ALL | wx.EXPAND, 5)
        sizer3.Add(wx.StaticText(self), 1, wx.ALL | wx.EXPAND, 5)
        sizer3.Add(hostCO2Label, 0, wx.ALL | wx.EXPAND, 5)
        #########################################################################
        # average version emission
        #########################################################################
        infoLabel = "Average Version Emission: "
        hostInfoLabel = wx.StaticText(co2Panel, label=infoLabel)
        co2Label ="%.15f" % avg_emission + " pounds"
        hostCO2Label = wx.StaticText(co2Panel, label=co2Label)
        sizer4 = wx.BoxSizer(wx.HORIZONTAL)
        sizer4.Add(hostInfoLabel, 0, wx.ALL | wx.EXPAND, 5)
        sizer4.Add(wx.StaticText(self), 1, wx.ALL | wx.EXPAND, 5)
        sizer4.Add(hostCO2Label, 0, wx.ALL | wx.EXPAND, 5)
        #########################################################################
        # total version emission
        #########################################################################
        infoLabel = "Total Version Emission: "
        hostInfoLabel = wx.StaticText(co2Panel, label=infoLabel)
        co2Label ="%.15f" % total_emission + " pounds"
        hostCO2Label = wx.StaticText(co2Panel, label=co2Label)
        sizer5 = wx.BoxSizer(wx.HORIZONTAL)
        sizer5.Add(hostInfoLabel, 0, wx.ALL | wx.EXPAND, 5)
        sizer5.Add(wx.StaticText(self), 1, wx.ALL | wx.EXPAND, 5)
        sizer5.Add(hostCO2Label, 0, wx.ALL | wx.EXPAND, 5)

        actualEmissionsBoxSizer.Add(sizer1, 0, wx.ALL | wx.EXPAND, 5)
        actualEmissionsBoxSizer.Add(sizer2, 0, wx.ALL | wx.EXPAND, 5)
        actualEmissionsBoxSizer.Add(sizer3, 0, wx.ALL | wx.EXPAND, 5)
        actualEmissionsBoxSizer.Add(sizer4, 0, wx.ALL | wx.EXPAND, 5)
        actualEmissionsBoxSizer.Add(sizer5, 0, wx.ALL | wx.EXPAND, 5)

        #########################################################################
        # MAIN LAYOUT
        #########################################################################

        mainSizer = wx.BoxSizer(wx.VERTICAL)
        mainSizer.Add(actualEmissionsBoxSizer, 0, wx.ALL | wx.EXPAND, 5)
        mainSizer.Add(wx.StaticText(self), 1, wx.ALL | wx.EXPAND, 5)

        co2Panel.SetSizer(mainSizer)
        co2Panel.Layout()
        co2Window.CentreOnScreen()
        co2Window.AddPanel(co2Panel)
        co2Window.SetWindowSize(600, 300)
        co2Window.Show()

    def _GetSelectedHost(self, simulator):

        host = None

        # get selected host name from combo
        hostName = self.hostsList.GetValue()

        # find host with the selected name
        for h in simulator.context.hosts:
            if h.original_name() == hostName:
                host = h
                break

        return host

    def _PopulateComboWithHostsNames(self, simulator):
        hostsNames = []
        [hostsNames.append(h.original_name()) for h in simulator.context.hosts if h.original_name() not in hostsNames]
        self.hostsList.Clear()
        self.hostsList.AppendItems(hostsNames)

    # ################
    # LAYOUT
    #################

    def _BuildHostsBoxAndSizer(self):
        """ """

        self.chooseHostLbl = wx.StaticText(self, label="Choose Host To See\nit's Total Cost:")
        self.hostsList = wx.ComboBox(self, style=wx.TE_READONLY, size=(200, -1))

        self.hostsBox = wx.StaticBox(self, label="Host(s)")
        self.hostsBoxSizer = wx.StaticBoxSizer(self.hostsBox, wx.HORIZONTAL)

        self.hostsBoxSizer.Add(self.chooseHostLbl, 0, wx.ALL | wx.EXPAND, 5)
        self.hostsBoxSizer.Add(wx.StaticText(self), 1, wx.ALL | wx.EXPAND, 5)
        self.hostsBoxSizer.Add(self.hostsList, 1, wx.ALL | wx.EXPAND | wx.ALIGN_RIGHT, 5)

        return self.hostsBox, self.hostsBoxSizer

    #################
    # REACTIONS
    #################

    def AddFinishedSimulation(self, simulator):
        """ """
        version = simulator.context.version
        self.versionsList.Append(version.name)
        self.versionSimulator[version.name] = simulator

    def OnVersionChanged(self, event):
        """ """
        versionName = self.versionsList.GetValue()
        simulator = self.versionSimulator[versionName]
        self._PopulateComboWithHostsNames(simulator)
        self.SetVersionsResultsVisibility(True)

    def RemoveAllSimulations(self):
        """ """
        self.versionsList.Clear()
        self.versionsList.SetValue("")
        self.versionSimulator = {}
        self.hostsList.Clear()
        self.hostsList.SetValue("")
        self.SetVersionsResultsVisibility(False)

    def SetVersionsResultsVisibility(self, visible):
        """ """
        widgets = []
        widgets.append(self.hostsList)
        widgets.append(self.co2Box)
        widgets.append(self.hostsBox)
        widgets.append(self.chooseHostLbl)

        for w in widgets:
            if visible:
                w.Show()
            else:
                w.Hide()

        self.Layout()


class MainResultsNotebook(wx.Notebook):
    def __init__(self, module, *args, **kwargs):
        wx.Notebook.__init__(self, *args, **kwargs)

        self.module = module

        il = wx.ImageList(24, 24)
        singleVersionImg = il.Add(wx.Bitmap(self.CreatePath4Resource('gogreen.png'), wx.BITMAP_TYPE_PNG))
        self.AssignImageList(il)

        self.oneVersionTab = SingleVersionPanel(self.module, self)
        self.AddPage(self.oneVersionTab, "Single Version")
        self.SetPageImage(0, singleVersionImg)
        self.oneVersionTab.Layout()

    def OnParsedModel(self):
        """ """
        self.oneVersionTab.RemoveAllSimulations()

    def OnSimulationFinished(self, simulator):
        """ """
        self.oneVersionTab.AddFinishedSimulation(simulator)

    def OnAllSimulationsFinished(self, simulators):
        """ """
        pass

    def CreatePath4Resource(self, resourceName):
        """
        @brief      creates and returns path to the
                    given file in the resource
                    ('assets') dir
        @return     path to the resource
        """
        tmp = os.path.split(os.path.dirname(__file__))
        # find last / character in path
        idx = tmp[0].rfind('/')
        # get substring - path for resource
        path = tmp[0][0:idx]
        return os.path.join(path, 'bin', 'assets', resourceName)


class ModuleGui(wx.EvtHandler):
    def __init__(self, module):
        """ """
        wx.EvtHandler.__init__(self)
        self.module = module
        self.mainResultNotebook = None

    def get_gui(self):
        if not getattr(self, '__gui', None):
            setattr(self, '__gui', ModuleGui(self))
        return getattr(self, '__gui', None)

    def get_name(self):
        return "Carbon Dioxide Emissions Analysis"

    def install_gui(self, simulator):
        """ Install module for gui simulation """
        self._install(simulator)
        return simulator

    def get_configuration_panel(self, parent):
        """ Returns WX panel with configuration controls. """

        panel = wx.Panel(parent)
        sizer = wx.BoxSizer(wx.VERTICAL)
        text = wx.StaticText(panel, label="Module does not need to be configured.")
        sizer.Add(text, 0, wx.ALL | wx.EXPAND, 5)
        text = wx.StaticText(panel, label="All result options will be available after results are calculated.")
        sizer.Add(text, 0, wx.ALL | wx.EXPAND, 5)
        panel.SetSizer(sizer)
        return panel

    def get_results_panel(self, parent):
        """
        Create main result panel existing from the beginning
        which will be extended when versions' simulations are finished.
        """
        self.mainResultNotebook = MainResultsNotebook(self.module, parent)
        return self.mainResultNotebook

    def on_finished_simulation(self, simulator):
        """ """
        self.mainResultNotebook.OnSimulationFinished(simulator)

    def on_finished_all_simulations(self, simulators):
        """
        Called once for all simulations after all of them are finished.
        """
        self.mainResultNotebook.OnAllSimulationsFinished(simulators)

    def on_parsed_model(self):
        """ """
        self.mainResultNotebook.OnParsedModel()


#!/usr/bin/env python

from aqopa import module
from aqopa.simulator.state import HOOK_TYPE_SIMULATION_FINISHED, HOOK_TYPE_PRE_INSTRUCTION_EXECUTION
from .gui import ModuleGui
from .console import PrintResultsHook
from .hook import PreInstructionHook

"""
@file       __init__.py
@brief      initial file for the qopanalysis module
@author     Katarzyna Mazur
"""

class Module(module.Module):

    def __init__(self):

        # all occured facts in a host, format: (simulator is the dict's key):
        # { simulator: {host0: [f1,f2, ... fn], host1: [f1,f2, ..., fm]} }
        self.occured_facts = {}

        # all facts in a model
        self.all_facts = []

        # qopanalysis params list in a host, format: (simulator is the dict's key):
        # { simulator: {host0: [[qop1, qop2, ..., qopn], ..., [qop1, qop2, ..., qopm]], host1: [[qop1, qop2, ..., qopk], ...,  [qop1, qop2, ..., qopx]]} }
        self.qopParams = {}

    def get_gui(self):
        if not getattr(self, '__gui', None):
            setattr(self, '__gui', ModuleGui(self))
        return getattr(self, '__gui', None)

    def _install(self, simulator):
        hook = PreInstructionHook(self, simulator)
        simulator.register_hook(HOOK_TYPE_PRE_INSTRUCTION_EXECUTION, hook)
        return simulator

    def install_console(self, simulator):
        """ Install module for console simulation """
        self._install(simulator)
        hook = PrintResultsHook(self, simulator)
        simulator.register_hook(HOOK_TYPE_SIMULATION_FINISHED, hook)
        return simulator

    def install_gui(self, simulator):
        """ Install module for gui simulation """
        self._install(simulator)
        return simulator

    def get_all_facts(self):
        """
        @brief returns a list which contains
        all the facts available in a model
        """
        return self.all_facts

    def set_all_facts(self, facts_list):
        """
        @brief sets all facts for the loaded
        model
        """
        self.all_facts = facts_list[:]

    def add_occured_fact(self, simulator, host, fact):
        """
        @brief adds a new, occured fact to the list
        of occured facts for the particular host
        present in the QoP-ML's model
        """
        # add a new simulator if not available yet
        if simulator not in self.occured_facts:
            self.occured_facts[simulator] = {}
        # add a new host if not available yet
        if host not in self.occured_facts[simulator] :
            self.occured_facts[simulator][host] = []
        # add a new fact for the host - but only if we
        # have not added it yet and if it is not empty
        if str(fact) not in self.occured_facts[simulator][host] and fact:
            # if the fact is actually a list of facts,
            if type(fact) is list:
                # add all the elements from the facts list
                for f in fact :
                    if str(f) not in self.occured_facts[simulator][host]:
                        self.occured_facts[simulator][host].append(str(f))
            else:
                self.occured_facts[simulator][host].append(str(fact))

    def get_occured_facts(self, simulator, host) :
        """
        @brief gets a list of all occured facts
        for the particular host present in the
        QoP-ML's model
        """
        if simulator not in self.occured_facts:
            self.occured_facts[simulator] = {}
        if host not in self.occured_facts[simulator]:
            self.occured_facts[simulator][host] = []
        return self.__make_list_flat(self.occured_facts[simulator][host])

    def get_qop_params(self):
        pass

    def add_qop_param(self):
        pass

    def __make_list_flat(self, l) :
        """
        @brief sometimes an element of the list
        might be a list too, so we need to flat
        the list given in the argument and return
        the oblate list
        """
        ans = []
        for i in l:
            if type(i) is list:
                ans = self.__make_list_flat(i)
            else:
                ans.append(i)
        return ans


'''
Created on 01-06-2013

@author: Damian Rusinek <damian.rusinek@gmail.com>
'''

from aqopa.model import MetricsServiceParam
from aqopa.model.parser.lex_yacc import LexYaccParserExtension


class Builder():
    
    def create_metrics_services_param_reputation(self, token):
        """
        metrics_services_param : SQLPARAN REPUTATION COLON ALGORITHM SQRPARAN
        """
        return MetricsServiceParam(token[2], token[4])


class MetricsParserExtension(LexYaccParserExtension):
    """
    Extension for parsing energy analysis module's metrics
    """
    
    def __init__(self):
        LexYaccParserExtension.__init__(self)
        
        self.builder = Builder()

    ##########################################
    #           RESERVED WORDS
    ##########################################

    def word_reputation(self, t):
        t.lexer.push_state('reputationprimhead')
        return t

    def word_algorithm(self, t):
        t.lexer.pop_state()
        return t

    ##########################################
    #                TOKENS
    ##########################################


    ##########################################
    #                RULES
    ##########################################
    
    def rule_metrics_services_param_reputation(self, t):
        """
        metrics_services_param : SQLPARAN REPUTATION COLON ALGORITHM SQRPARAN
        """
        t[0] = self.builder.create_metrics_services_param_reputation(t)
    
    def _extend(self):
        """ """
        self.parser.add_state('reputationprimhead', 'inclusive')

        self.parser.add_reserved_word('reputation', 'REPUTATION', func=self.word_reputation, state='metricsprimhead',
                                      case_sensitive=True)

        self.parser.add_token('COLON', r':', states=['reputationprimhead'])
        self.parser.add_reserved_word('algorithm', 'ALGORITHM', func=self.word_algorithm, state='reputationprimhead',
                                      case_sensitive=True)

        self.parser.add_rule(self.rule_metrics_services_param_reputation)


class ModelParserExtension(LexYaccParserExtension):
    """
    Extension for parsing energy analysis module's metrics
    """

    def __init__(self, module):
        LexYaccParserExtension.__init__(self)

        self.module = module
        self.builder = Builder()

        self.open_blocks_cnt = 0

    ##########################################
    #                TOKENS
    ##########################################

    def token_reputation(self, t):
        r'reputation'
        t.lexer.push_state('reputation')
        return t

    def token_blockopen(self, t):
        r"\{"
        self.open_blocks_cnt += 1
        return t

    def token_blockclose(self, t):
        r"\}"
        self.open_blocks_cnt -= 1
        if self.open_blocks_cnt == 0:
            t.lexer.pop_state()
        return t

    ##########################################
    #                RULES
    ##########################################

    def reputation_specification(self, t):
        """
        module_specification : REPUTATION BLOCKOPEN reputation_init_vars reputation_algorithms BLOCKCLOSE
        """
        pass

    def reputation_algorithms(self, t):
        """
        reputation_algorithms : reputation_algorithm
                            | reputation_algorithms reputation_algorithm
        """
        pass

    def reputation_algorithm(self, t):
        """
        reputation_algorithm : ALGORITHM IDENTIFIER LPARAN identifiers_list RPARAN BLOCKOPEN reputation_instructions BLOCKCLOSE
        """
        self.module.algorithms[t[2]] = {
            'parameters': t[4],
            'instructions': t[7],
        }

    def reputation_init_vars(self, t):
        """
        reputation_init_vars : reputation_init_var
                            | reputation_init_vars reputation_init_var
        """
        pass

    def reputation_init_var(self, t):
        """
        reputation_init_var : HASH IDENTIFIER EQUAL number SEMICOLON
        """
        self.module.init_vars[t[2]] = t[4]

    def reputation_instructions(self, t):
        """
        reputation_instructions : reputation_instruction
                                | reputation_instructions reputation_instruction
        """
        if len(t) == 2:
            t[0] = []
            t[0].append(t[1])
        else:
            t[0] = t[1]
            t[0].append(t[2])


    def reputation_instruction(self, t):
        """
        reputation_instruction : reputation_instruction_assignment SEMICOLON
                                | reputation_instruction_conditional
                                | reputation_instruction_conditional SEMICOLON
        """
        t[0] = t[1]

    def reputation_instruction_assignment(self, t):
        """
        reputation_instruction_assignment : IDENTIFIER EQUAL reputation_expression
        """
        t[0] = {'type': 'assignment', 'identifier': t[1], 'expression': t[3]}

    def reputation_expression_operations(self, t):
        """
        reputation_expression : reputation_expression PLUS reputation_expression
                            | reputation_expression MINUS reputation_expression
                            | reputation_expression TIMES reputation_expression
                            | reputation_expression DIVIDE reputation_expression
        """
        t[0] = ''.join(t[1:])

    def reputation_expression_values(self, t):
        """
        reputation_expression : number
                            | IDENTIFIER
        """
        t[0] = str(t[1])

    def reputation_expression_uminus(self, t):
        """
        reputation_expression : MINUS reputation_expression %prec UMINUS
        """
        t[0] = ''.join(t[1:2])

    def reputation_expression_paran(self, t):
        """
        reputation_expression : LPARAN reputation_expression RPARAN
        """
        t[0] = ''.join(t[1:])

    def reputation_instruction_conditional(self, t):
        """
        reputation_instruction_conditional : IF LPARAN reputation_expression_conditional RPARAN BLOCKOPEN reputation_instructions BLOCKCLOSE
                                        | IF LPARAN reputation_expression_conditional RPARAN BLOCKOPEN reputation_instructions BLOCKCLOSE ELSE BLOCKOPEN reputation_instructions BLOCKCLOSE
        """

        if_instruction = {'type': 'if', 'condition': t[3], 'true_instructions': t[6], 'false_instructions': []}
        if len(t) > 8:
            if_instruction['false_instructions'] = t[10]
        t[0] = if_instruction

    def reputation_expression_conditional_comparison(self, t):
        """
        reputation_expression_conditional : reputation_expression EQUAL EQUAL reputation_expression
                                        | reputation_expression EXCLAMATION EQUAL reputation_expression
                                        | reputation_expression GREATER reputation_expression
                                        | reputation_expression GREATER EQUAL reputation_expression
                                        | reputation_expression SMALLER reputation_expression
                                        | reputation_expression SMALLER EQUAL reputation_expression
                                        | reputation_expression_conditional AND AND reputation_expression_conditional
                                        | reputation_expression_conditional OR OR reputation_expression_conditional
        """
        t[0] = ''.join(t[1:])

    def reputation_expression_conditional_operators(self, t):
        """
        reputation_expression_conditional : USED LPARAN IDENTIFIER RPARAN
                                        | EXISTS LPARAN IDENTIFIER RPARAN
        """
        t[0] = ''.join(t[1:])

    def reputation_expression_conditional_paran(self, t):
        """
        reputation_expression_conditional : LPARAN reputation_expression_conditional RPARAN
        """
        t[0] = ''.join(t[1:])

    def _extend(self):
        """ """
        self.parser.add_state('reputation', 'inclusive')

        self.parser.add_token('REPUTATION', func=self.token_reputation, states=['modules'])
        self.parser.add_reserved_word('algorithm', 'ALGORITHM', state='reputation', case_sensitive=True)
        self.parser.add_reserved_word('if', 'IF', state='reputation', case_sensitive=True)
        self.parser.add_reserved_word('else', 'ELSE', state='reputation', case_sensitive=True)
        self.parser.add_reserved_word('used', 'USED', state='reputation', case_sensitive=True)
        self.parser.add_reserved_word('exists', 'EXISTS', state='reputation', case_sensitive=True)

        self.parser.add_token('BLOCKOPEN', func=self.token_blockopen, states=['reputation'])
        self.parser.add_token('BLOCKCLOSE', func=self.token_blockclose, states=['reputation'])

        self.parser.add_token('PLUS', r'\+', states=['reputation'])
        self.parser.add_token('MINUS', r'\-', states=['reputation'])
        self.parser.add_token('TIMES', r'\*', states=['reputation'])
        self.parser.add_token('DIVIDE', r'/', states=['reputation'])

        self.parser.add_token('HASH', r"\#", states=['reputation'])
        self.parser.add_token('GREATER', r'\>', states=['reputation'])
        self.parser.add_token('SMALLER', r'\<', states=['reputation'])
        self.parser.add_token('EXCLAMATION', r'\!', states=['reputation'])
        self.parser.add_token('AND', r'\&', states=['reputation'])
        self.parser.add_token('OR', r'\|', states=['reputation'])

        self.parser.add_precedence(['PLUS', 'MINUS'], 'left')
        self.parser.add_precedence(['TIMES', 'DIVIDE'], 'left')
        self.parser.add_precedence(['UMINUS'], 'right')

        # Rules

        self.parser.add_rule(self.reputation_specification)
        self.parser.add_rule(self.reputation_algorithms)
        self.parser.add_rule(self.reputation_algorithm)
        self.parser.add_rule(self.reputation_init_vars)
        self.parser.add_rule(self.reputation_init_var)
        self.parser.add_rule(self.reputation_instructions)
        self.parser.add_rule(self.reputation_instruction)
        self.parser.add_rule(self.reputation_instruction_assignment)
        self.parser.add_rule(self.reputation_expression_operations)
        self.parser.add_rule(self.reputation_expression_paran)
        self.parser.add_rule(self.reputation_expression_uminus)
        self.parser.add_rule(self.reputation_expression_values)
        self.parser.add_rule(self.reputation_instruction_conditional)
        self.parser.add_rule(self.reputation_expression_conditional_comparison)
        self.parser.add_rule(self.reputation_expression_conditional_operators)
        self.parser.add_rule(self.reputation_expression_conditional_paran)


'''
Created on 01-06-2013

@author: Damian Rusinek <damian.rusinek@gmail.com>
'''

from aqopa.model import MetricsServiceParam
from aqopa.model.parser.lex_yacc import LexYaccParserExtension


class Builder():
    
    def create_metrics_services_param_time(self, token):
        """
        metrics_services_param : SQLPARAN TIME COLON metrics_services_time_type LPARAN metrics_services_time_unit RPARAN SQRPARAN
                                | SQLPARAN TIME COLON metrics_services_time_two_params_type LPARAN metrics_services_exact_time_unit COMMA metrics_services_size_unit RPARAN SQRPARAN
        """
        if len(token) == 9:
            return MetricsServiceParam(token[2], token[4], token[6])
        else:
            return MetricsServiceParam(token[2], token[4], (token[6], token[8]))


class ModelParserExtension(LexYaccParserExtension):
    """
    Extension for timeanalysis module communications time
    """

    #######################
    #  Communication Time
    #######################

    def time_default_parameter(self, t):
        """
        medium_default_parameter : TIME_DEFAULT_PARAMETER EQUAL comm_time_value
        """
        t[0] = {'default_time': t[3]}

    def topology_rule_time_parameter(self, t):
        """
        topology_rule_parameter : TIME_PARAMETER EQUAL comm_time_value
        """
        t[0] = {'time': t[3]}

    def comm_time_value(self, t):
        """
        comm_time_value : comm_time_metric
                            | comm_time_algorithm
        """
        t[0] = t[1]

    def comm_time_metric(self, t):
        """
        comm_time_metric : number comm_time_metric_unit
        """
        t[0] = {
            'type': 'metric',
            'value': t[1],
            'unit': t[2]
        }

    def comm_time_algorithm(self, t):
        """
        comm_time_algorithm : IDENTIFIER SQLPARAN comm_time_metric_unit SQRPARAN
                        | IDENTIFIER
        """
        unit = 'ms'
        if len(t) == 5:
            unit = t[3]
        t[0] = {
            'type': 'algorithm',
            'name': t[1],
            'unit': unit
        }

    def comm_time_metric_unit(self, t):
        """
        comm_time_metric_unit : MS
                                | MSPBIT
                                | MSPBYTE
                                | KBYTEPS
                                | MBYTEPS
        """
        t[0] = t[1]

    def _extend(self):
        """ """

        self.parser.add_reserved_word('ms', 'MS', state='communication', case_sensitive=True)
        self.parser.add_reserved_word('mspb', 'MSPBIT', state='communication', case_sensitive=True)
        self.parser.add_reserved_word('mspB', 'MSPBYTE', state='communication', case_sensitive=True)
        self.parser.add_reserved_word('kbps', 'KBYTEPS', state='communication', case_sensitive=True)
        self.parser.add_reserved_word('mbps', 'MBYTEPS', state='communication', case_sensitive=True)
        self.parser.add_reserved_word('default_time', 'TIME_DEFAULT_PARAMETER', state='communication',)
        self.parser.add_reserved_word('time', 'TIME_PARAMETER', state='communication',)

        self.parser.add_rule(self.time_default_parameter)
        self.parser.add_rule(self.topology_rule_time_parameter)
        self.parser.add_rule(self.comm_time_value)
        self.parser.add_rule(self.comm_time_metric)
        self.parser.add_rule(self.comm_time_metric_unit)
        self.parser.add_rule(self.comm_time_algorithm)


class ConfigParserExtension(LexYaccParserExtension):
    """
    Extension for timeanalysis module communications time
    """

    #######################
    #  Communication Time
    #######################

    def version_time_default_parameter(self, t):
        """
        version_medium_default_parameter : TIME_DEFAULT_PARAMETER EQUAL version_comm_time_value
        """
        t[0] = {'default_time': t[3]}

    def version_topology_rule_time_parameter(self, t):
        """
        version_topology_rule_parameter : TIME_PARAMETER EQUAL version_comm_time_value
        """
        t[0] = {'time': t[3]}

    def version_comm_time_value(self, t):
        """
        version_comm_time_value : version_comm_time_metric
                            | version_comm_time_algorithm
        """
        t[0] = t[1]

    def version_comm_time_metric(self, t):
        """
        version_comm_time_metric : number version_comm_time_metric_unit
        """
        t[0] = {
            'type': 'metric',
            'value': t[1],
            'unit': t[2]
        }

    def version_comm_time_algorithm(self, t):
        """
        version_comm_time_algorithm : IDENTIFIER SQLPARAN version_comm_time_metric_unit SQRPARAN
                                | IDENTIFIER
        """
        unit = 'ms'
        if len(t) == 5:
            unit = t[3]
        t[0] = {
            'type': 'algorithm',
            'name': t[1],
            'unit': unit
        }

    def version_comm_time_metric_unit(self, t):
        """
        version_comm_time_metric_unit : MS
                                | MSPBIT
                                | MSPBYTE
                                | KBYTEPS
                                | MBYTEPS
        """
        t[0] = t[1]

    def _extend(self):
        """ """

        self.parser.add_reserved_word('ms', 'MS', state='versioncommunication', case_sensitive=True)
        self.parser.add_reserved_word('mspb', 'MSPBIT', state='versioncommunication', case_sensitive=True)
        self.parser.add_reserved_word('mspB', 'MSPBYTE', state='versioncommunication', case_sensitive=True)
        self.parser.add_reserved_word('kbps', 'KBYTEPS', state='versioncommunication', case_sensitive=True)
        self.parser.add_reserved_word('mbps', 'MBYTEPS', state='versioncommunication', case_sensitive=True)
        self.parser.add_reserved_word('default_time', 'TIME_DEFAULT_PARAMETER', state='versioncommunication',)
        self.parser.add_reserved_word('time', 'TIME_PARAMETER', state='versioncommunication',)

        self.parser.add_rule(self.version_time_default_parameter)
        self.parser.add_rule(self.version_topology_rule_time_parameter)
        self.parser.add_rule(self.version_comm_time_value)
        self.parser.add_rule(self.version_comm_time_metric)
        self.parser.add_rule(self.version_comm_time_metric_unit)
        self.parser.add_rule(self.version_comm_time_algorithm)

class MetricsParserExtension(LexYaccParserExtension):
    """
    Extension for parsing timeanalysis module metrics
    """
    
    def __init__(self):
        LexYaccParserExtension.__init__(self)
        
        self.builder = Builder()


    #######################
    #    Metrics Time
    #######################
    
    def metrics_services_param_time(self, t):
        """
        metrics_services_param : SQLPARAN TIME COLON metrics_services_time_type LPARAN metrics_services_time_unit RPARAN SQRPARAN
                                | SQLPARAN TIME COLON metrics_services_time_two_params_type LPARAN metrics_services_exact_time_unit COMMA metrics_services_size_unit RPARAN SQRPARAN
        """
        t[0] = self.builder.create_metrics_services_param_time(t)
    
    def metrics_services_time_one_param_type(self, t):
        """
        metrics_services_time_type : EXACT
                                | RANGE
                                | ALGORITHM
        """
        t[0] = t[1].lower()

    def metrics_services_time_two_params_type(self, t):
        """
        metrics_services_time_two_params_type : BLOCK
        """
        t[0] = t[1].lower()
    
    def metrics_services_time_unit(self, t):
        """
        metrics_services_time_unit : S
                                | MS
                                | MSPBIT
                                | MSPBYTE
                                | KBYTEPS
                                | MBYTEPS
        """
        t[0] = t[1]

    def metrics_services_exact_time_unit(self, t):
        """
        metrics_services_exact_time_unit : S
                                        | MS
        """
        t[0] = t[1].lower()

    def metrics_services_size_unit(self, t):
        """
        metrics_services_size_unit : B
                                | b
        """
        t[0] = t[1]
    
    def _extend(self):
        """ """

        self.parser.add_token('COMMA', r'\,', states=['metricsprimhead'])

        self.parser.add_reserved_word('time', 'TIME', state='metricsprimhead', case_sensitive=False)
        self.parser.add_reserved_word('exact', 'EXACT', state='metricsprimhead', case_sensitive=False)
        self.parser.add_reserved_word('range', 'RANGE', state='metricsprimhead', case_sensitive=False)
        self.parser.add_reserved_word('s', 'S', state='metricsprimhead', case_sensitive=True)
        self.parser.add_reserved_word('ms', 'MS', state='metricsprimhead', case_sensitive=True)
        self.parser.add_reserved_word('mspb', 'MSPBIT', state='metricsprimhead', case_sensitive=True)
        self.parser.add_reserved_word('mspB', 'MSPBYTE', state='metricsprimhead', case_sensitive=True)
        self.parser.add_reserved_word('kbps', 'KBYTEPS', state='metricsprimhead', case_sensitive=True)
        self.parser.add_reserved_word('mbps', 'MBYTEPS', state='metricsprimhead', case_sensitive=True)
        self.parser.add_reserved_word('algorithm', 'ALGORITHM', state='metricsprimhead', case_sensitive=True)

        self.parser.add_reserved_word('block', 'BLOCK', state='metricsprimhead', case_sensitive=False)
        self.parser.add_reserved_word('b', 'b', state='metricsprimhead', case_sensitive=True)
        self.parser.add_reserved_word('B', 'B', state='metricsprimhead', case_sensitive=True)
        
        self.parser.add_rule(self.metrics_services_param_time)
        self.parser.add_rule(self.metrics_services_time_one_param_type)
        self.parser.add_rule(self.metrics_services_time_two_params_type)
        self.parser.add_rule(self.metrics_services_time_unit)
        self.parser.add_rule(self.metrics_services_exact_time_unit)
        self.parser.add_rule(self.metrics_services_size_unit)


#!/usr/bin/env python

import wx
import os

# AQoPA imports
import aqopa

# AQoPA gui imports
from aqopa.gui.models_lib_gui import LibraryFrame, EVT_MODEL_SELECTED
from aqopa.gui.main_notebook_gui import MainNotebook

"""
@file       main_frame_gui.py
@brief      GUI for the main frame (the one with the tabs on it)
@author     Damian Rusinek <damian.rusinek@gmail.com>
@date       created on 05-09-2013 by Damian Rusinek
@date       edited on 07-05-2014 by Katarzyna Mazur (visual improvements)
"""

class MainFrame(wx.Frame):
    """ """
    def __init__(self, *args, **kwargs):
        wx.Frame.__init__(self, *args, **kwargs)

        ###########
        # MENUBAR
        ###########

        # create menubar
        menuBar = wx.MenuBar()

        #create main menu, lets call it 'file' menu
        fileMenu = wx.Menu()
        # create menu item = about AQoPA
        item = wx.MenuItem(fileMenu, wx.NewId(), u"&About AQoPA\tCTRL+I")
        item.SetBitmap(wx.Bitmap(self.CreatePath4Resource('about.png')))
        fileMenu.AppendItem(item)
        self.Bind(wx.EVT_MENU, self.OnAbout, item)
        fileMenu.AppendSeparator()
        # create menu item = quit AQoPa
        item = wx.MenuItem(fileMenu, wx.NewId(), u"&Quit\tCTRL+Q")
        item.SetBitmap(wx.Bitmap(self.CreatePath4Resource('exit.png')))
        fileMenu.AppendItem(item)
        self.Bind(wx.EVT_MENU, self.OnQuit, item)
        # add 'file' menu to the menubar
        menuBar.Append(fileMenu, "&Menu")

        # create library menu, here u can find modules library
        libraryMenu = wx.Menu()
        # create models menu item
        item = wx.MenuItem(libraryMenu, wx.NewId(), u"&Browse models\tCTRL+M")
        item.SetBitmap(wx.Bitmap(self.CreatePath4Resource('models_lib.png')))
        libraryMenu.AppendItem(item)
        self.Bind(wx.EVT_MENU, self.OnBrowseModels, item)

        # create metric menu item
        item = wx.MenuItem(libraryMenu, wx.NewId(), u"&Browse metrics\tCTRL+F")
        item.SetBitmap(wx.Bitmap(self.CreatePath4Resource('metrics.png')))
        libraryMenu.AppendItem(item)

        # add 'library' menu to the menubar
        menuBar.Append(libraryMenu, "&Library")

        self.SetMenuBar(menuBar)

        ###################
        # SIZERS & EVENTS
        ###################

        self.mainNotebook = MainNotebook(self)

        logoPanel = wx.Panel(self)
        pic = wx.StaticBitmap(logoPanel)
        pic.SetBitmap(wx.Bitmap(self.CreatePath4Resource('logo.png')))

        sizer = wx.BoxSizer(wx.HORIZONTAL)
        sizer.Add(wx.StaticText(self), 1, wx.EXPAND, 5)
        sizer.Add(logoPanel, 0, wx.RIGHT| wx.ALL|wx.EXPAND, 5)
        s2 = wx.BoxSizer(wx.VERTICAL)
        s2.Add(sizer, 0, wx.LEFT| wx.ALL|wx.EXPAND, 5)
        s2.Add(self.mainNotebook, 1, wx.ALL|wx.EXPAND, 5)
        self.SetSizer(s2)

        self.SetIcon(wx.Icon(self.CreatePath4Resource('app_logo.png'), wx.BITMAP_TYPE_PNG))
        self.SetMinSize(wx.Size(900, 700))
        self.CenterOnScreen()
        self.Layout()

    def CreatePath4Resource(self, resourceName):
        """
        @brief      creates and returns path to the
                    given file in the resource
                    ('assets') dir
        @return     path to the resource
        """
        tmp = os.path.split(os.path.dirname(__file__))
        return os.path.join(tmp[0], 'bin', 'assets', resourceName)

    def OnQuit(self, event=None):
        """
        @brief  closes the application
        """
        self.Close()

    def OnBrowseModels(self, event=None):
        """
        @brief  shows the library frame (models library window)
        """
        libraryFrame = LibraryFrame(self, title="Models Library")
        libraryFrame.Show(True)
        libraryFrame.CentreOnParent()
        #libraryFrame.Maximize(True)
        libraryFrame.Bind(EVT_MODEL_SELECTED, self.OnLibraryModelSelected)

    def OnLibraryModelSelected(self, event):
        """ """
        self.mainNotebook.SetModelData(event.model_data)
        self.mainNotebook.SetMetricsData(event.metrics_data)
        self.mainNotebook.SetVersionsData(event.versions_data)
        # set filenames on GUI
        self.mainNotebook.modelTab.SetFilenameOnGUI(event.filenames['model'])
        self.mainNotebook.metricsTab.SetFilenameOnGUI(event.filenames['metrics'])
        self.mainNotebook.versionsTab.SetFilenameOnGUI(event.filenames['versions'])

    def OnAbout(self, event=None):
        """ Show about info """

        description = """AQoPA stands for Automated Quality of Protection Analysis Tool
        for QoPML models."""

        licence = """AQoPA is free software; you can redistribute
it and/or modify it under the terms of the GNU General Public License as
published by the Free Software Foundation; either version 2 of the License,
or (at your option) any later version.

AQoPA is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE."""

        logo_filepath = self.CreatePath4Resource('logo.png')

        info = wx.AboutDialogInfo()
        info.SetIcon(wx.Icon(logo_filepath, wx.BITMAP_TYPE_PNG))
        info.SetName('AQoPA')
        info.SetVersion(aqopa.VERSION)
        info.SetDescription(description)
        info.SetCopyright('(C) 2013 QoPML Project')
        info.SetWebSite('http://www.qopml.org')
        info.SetLicence(licence)
        info.AddDeveloper('Damian Rusinek')
        info.AddDocWriter('Damian Rusinek')
        info.AddArtist('QoPML Project')
        info.AddTranslator('Damian Rusinek')

        wx.AboutBox(info)


from aqopa.model import original_name, CallFunctionExpression,\
    IdentifierExpression
from aqopa.simulator.error import RuntimeException

###############################
#    PREDEFINED FUNCTIONS
###############################


# Function ID


def _predefined_id_function__populate(call_function_expression, host, populator, context=None):
    existing_host_name = getattr(call_function_expression, '_host_name', None)
    if existing_host_name is not None:
        return call_function_expression
    arguments = call_function_expression.arguments
    if len(arguments) == 0:
        expr_host_name = host.name
    else:
        expr_host_name = arguments[0].identifier
        if expr_host_name == original_name(expr_host_name):
            expr_host_name += ".0"
    setattr(call_function_expression, '_host_name', expr_host_name)
    return call_function_expression


def _predefined_id_function__are_equal(left, right):
    if left.function_name != right.function_name:
        return False
    left_host_name = getattr(left, '_host_name', None)
    right_host_name = getattr(right, '_host_name', None)
    if left_host_name is None or right_host_name is None:
        return False
    return left_host_name == right_host_name


# Function routing_next


def _predefined_routing_next_function__populate(call_function_expression, host, populator, context=None):

    topology_name = call_function_expression.arguments[0].identifier

    receiver_function_id_expression = populator.populate(call_function_expression.arguments[1], host)
    receiver_function_id_expression = _predefined_id_function__populate(receiver_function_id_expression, 
                                                                        host, populator)
    receiver_name = getattr(receiver_function_id_expression, '_host_name')
    
    receiver = None
    for h in context.hosts:
        if h.name == receiver_name:
            receiver = h
            break
         
    sender = host
    sender_name = sender.name
    if len(call_function_expression.arguments) > 2:
        sender_function_id_expression = populator.populate(call_function_expression.arguments[2], host)
        sender_function_id_expression = _predefined_id_function__populate(sender_function_id_expression, 
                                                                          host, populator)
        sender_name = getattr(sender_function_id_expression, '_host_name')
        if sender_name != sender.name:
            sender = None
            for h in context.hosts:
                if h.name == sender_name:
                    sender = h
                    break
    
    if sender is None:
        raise RuntimeException("Host '%s' undefined." % sender_name)
    if receiver is None:
        raise RuntimeException("Host '%s' undefined." % receiver_name)

    if sender == receiver:
        next_host = receiver
    else:
        next_host = context.channels_manager.get_router().get_next_hop_host(topology_name, sender, receiver)
        if next_host is None:
            raise RuntimeException("The route from host '%s' to host '%s' cannot be found."
                                   % (sender_name, receiver_name))
    # DEBUG
    # print host.name, ': ', unicode(call_function_expression), ' -> ', next_host.name
    # DEBUG
            
    id_function = CallFunctionExpression('id', arguments=[IdentifierExpression(next_host.name)])
    setattr(id_function, '_host_name', next_host.name)
    return id_function


###############################
#    PREDEFINED MANAGEMENT
###############################

class FunctionsManager():
    
    def __init__(self, context):
        self.context = context
        self.functions_map = {
                'id': {
                    'populate': _predefined_id_function__populate,
                    'are_equal': _predefined_id_function__are_equal
                },
                'routing_next': {
                    'populate': _predefined_routing_next_function__populate,
                    'are_equal': None
                }
            }

    def is_function_predefined(self, function_name):
        return function_name in self.functions_map
    
    def populate_call_function_expression_result(self, call_function_expression, host, populator):
        fun_name = call_function_expression.function_name
        if not self.is_function_predefined(fun_name):
            return call_function_expression
        return self.functions_map[fun_name]['populate'](call_function_expression, host, populator, context=self.context)

    def are_equal_call_function_expressions(self, left, right):
        if not isinstance(left, CallFunctionExpression) or not isinstance(right, CallFunctionExpression):
            raise RuntimeException("Trying to compare predefined functions expressions, but they are not.")
        return self.functions_map[left.function_name]['are_equal'](left, right)
    



#!/usr/bin/env python

import sys
from aqopa.simulator.state import Hook

"""
@file       console.py
@author     Katarzyna Mazur
"""

class PrintResultsHook(Hook):

    def __init__(self, module, simulator, output_file=sys.stdout):
        """ """
        self.module = module
        self.simulator = simulator
        self.output_file = output_file

    def execute(self, context, **kwargs):
        """ """

        self.output_file.write('-'*80)
        self.output_file.write('\n')
        self.output_file.write('Module\tQoP Analysis')
        self.output_file.write('\n')
        self.output_file.write('Version\t%s\n\n' % self.simulator.context.version.name)

        self.output_file.write('All occured facts:\t{0}\n'.format(self.module.get_all_facts()))

        self.output_file.write("\nActual facts:\n")
        for host in context.hosts:
            self.output_file.write('Host:\t{0}\tFacts: {1:}\t\n'.format(host.name, str(self.module.get_occured_facts(self.simulator, host))))


#!/usr/bin/env python

from .qop_param import QoPParameter
from aqopa.simulator.state import Hook, ExecutionResult
from aqopa.model import AssignmentInstruction,\
    CallFunctionInstruction, IfInstruction, WhileInstruction,\
    CommunicationInstruction, CallFunctionExpression, TupleExpression, ComparisonExpression, \
    IdentifierExpression

"""
@file       hook.py
@author     Katarzyna Mazur
"""

class PreInstructionHook(Hook):

    """
    Execution hook executed before default core execution of each instruction.
    Returns execution result.
    """

    # its static, cause we need to have the list for EVERY
    # simulator and its hosts, right?
    all_facts = []

    def __init__(self, module, simulator):
        self.module = module
        self.simulator = simulator

    def execute(self, context, **kwargs):
        instruction = context.get_current_instruction()

        # temp - test me!
        # if instruction.__class__ is CallFunctionInstruction:
        #     print "function_name: "+str(instruction.function_name)
        #     print "arguments: "+str(instruction.arguments)
        #     print "qop_arguments: "+str(instruction.qop_arguments)
        #     for i in instruction.arguments:
        #         print "qop_argument:"+str(i.identifier)

        if instruction.__class__ not in [AssignmentInstruction, CallFunctionInstruction,
                                     IfInstruction, WhileInstruction]:
            return

        self._update_occured_facts(context)
        self._update_all_facts(context)
        return ExecutionResult()

    def _update_occured_facts(self, context):
        """
        Update all facts in context according to current instruction.
        """

        # get current instruction
        instruction = context.get_current_instruction()

        # check the instruction type and extract the expression from it
        if isinstance(instruction, AssignmentInstruction):
            expression = instruction.expression
        elif isinstance(instruction, CallFunctionInstruction):
            expression = CallFunctionExpression(instruction.function_name, instruction.arguments, instruction.qop_arguments)
        else:
            expression = instruction.condition

        # Return details for each expression in instruction
        # In some instruction may be more expressions (tuple, nested call function)
        fact = self._get_occured_facts_details_for_expression(context, expression)

        # add fact to the module's fact list
        self.module.add_occured_fact(self.simulator, context.get_current_host(), fact)

    def _get_occured_facts_details_for_expression(self, context, expression):
        if isinstance(expression, TupleExpression):
            return self._get_occured_facts_details_for_tuple_expression(context, expression)
        elif isinstance(expression, CallFunctionExpression):
            return self._get_occured_facts_details_for_simple_expression(context, expression)
        elif isinstance(expression, ComparisonExpression):
            return self._get_occured_facts_details_for_comparison_expression(context, expression)
        return []

    def _get_occured_facts_details_for_tuple_expression(self, context, expression):
        qop_args = []
        for i in range(0, len(expression.elements)):
            qop_arg = self._get_occured_facts_details_for_expression(context, expression.elements[i])
            if type(qop_arg) is list :
                qop_args.extend(qop_arg)
            else:
                qop_args.append(qop_arg)
        return qop_args

    def _get_occured_facts_details_for_simple_expression(self, context, expression):
        qop_args = []
        for expr in expression.qop_arguments:
            #e = self._get_occured_facts_details_for_expression(context, expr)
            qop_args.append(str(expr))
        return qop_args

    def _get_occured_facts_details_for_comparison_expression(self, context, expression):
        qop_args = []
        l = self._get_occured_facts_details_for_expression(context, expression.left)
        r = self._get_occured_facts_details_for_expression(context, expression.right)
        if type(l) is list :
            qop_args.extend(l)
        else:
            qop_args.append(l)
        if type(r) is list :
            qop_args.extend(r)
        else:
            qop_args.append(r)
        return qop_args

    def _update_all_facts(self, context):
        # make a list from all the facts within the simulator and its hosts,
        # simply concatenate all the facts from all the simulators and its all
        # hosts into a one, big list of facts available within the loaded model
        facts = self.module.get_occured_facts(self.simulator, context.get_current_host())
        for fact in facts:
            if fact not in PreInstructionHook.all_facts:
                PreInstructionHook.all_facts.append(fact)
        self.module.set_all_facts(PreInstructionHook.all_facts)


#!/usr/bin/env python

"""
@file       qop_param.py
@brief      a class that describes a single QoP paramater
@author     Katarzyna Mazur
"""

class QoPParameter():

    # static params counter
    qop_param_num = 1

    def __init__(self, qop_param_name="", fun_name="", val=""):
        # those values can describe the QoP parameters
        self.qop_label = "qopanalysis" + QoPParameter.qop_param_num
        self.qop_param_name = qop_param_name
        self.fun_name = fun_name
        self.val = val

        QoPParameter.qop_param_num += 1

    def __str__(self):
        return "QoP[label=%s, name=%s, function=%s, value=%s]" % (self.qop_label, self.qop_param_name, self.fun_name, self.val)

    def __repr__(self):
        return "QoP[label=%s, name=%s, function=%s, value=%s]" % (self.qop_label, self.qop_param_name, self.fun_name, self.val)

    def __unicode__(self):
        return u"QoP[label=%s, name=%s, function=%s, value=%s]" % (self.qop_label, self.qop_param_name, self.fun_name, self.val)


'''
Created on 22-04-2013

@author: Damian Rusinek <damian.rusinek@gmail.com>
'''

from aqopa.model.parser.lex_yacc import LexYaccParserExtension
from aqopa.model import Channel, TopologyRuleHost, TopologyRule,\
    AlgWhile, AlgCallFunction, AlgIf, AlgReturn, AlgAssignment


class Builder():
    """
    Builder for store objects
    """
    


class ModelParserExtension(LexYaccParserExtension):
    """
    Extension for parsing functions
    """
    
    def __init__(self):
        LexYaccParserExtension.__init__(self)
        self.builder = Builder()
        self.open_blocks_cnt = 0

    ##########################################
    #           RESERVED WORDS
    ##########################################
    
    def word_communication_specification(self, t):
        t.lexer.push_state('communication')
        return t
    
    ##########################################
    #                TOKENS
    ##########################################

    def token_block_open(self, t):
        r'{'
        self.open_blocks_cnt += 1
        return t

    def token_block_close(self, t):
        r'}'
        self.open_blocks_cnt -= 1
        if self.open_blocks_cnt == 0:
            t.lexer.pop_state()
        return t
    
    ##########################################
    #                RULES
    ##########################################
    
    def communication_specification(self, t):
        """
        specification : COMMUNICATION_SPECIFICATION BLOCKOPEN comm_specifications BLOCKCLOSE
        """
        pass
    
    
    def comm_specifications(self, t):
        """
        comm_specifications : comm_specification
                    | comm_specifications comm_specification
        """
        pass

    #  Medium

    def medium_specification(self, t):
        """
        comm_specification : MEDIUM_SPECIFICATION SQLPARAN IDENTIFIER SQRPARAN BLOCKOPEN medium_elements BLOCKCLOSE
        """
        self.parser.store.mediums[t[3]] = t[6]

    def medium_elements(self, t):
        """
        medium_elements : medium_default_parameters medium_topology
        """
        t[0] = {
            'topology': t[2],
            'default_parameters': t[1],
        }

    def medium_default_parameters(self, t):
        """
        medium_default_parameters : medium_default_parameter SEMICOLON
                                | medium_default_parameters medium_default_parameter SEMICOLON
        """
        t[0] = t[1]
        if len(t) == 4:
            t[0].update(t[2])

    def quality_default_parameter(self, t):
        """
        medium_default_parameter : QUALITY_DEFAULT_PARAMETER EQUAL number
        """
        t[0] = {'default_q': t[3]}

    #  Topology
    
    def medium_topology(self, t):
        """
        medium_topology : TOPOLOGY_SPECIFICATION BLOCKOPEN topology_rules_list BLOCKCLOSE
        """
        t[0] = {'rules': t[3]}

    def topology_rules_list(self, t):
        """
        topology_rules_list : topology_rule
                        | topology_rules_list topology_rule
        """
        if len(t) == 3:
            t[0] = t[1]
            t[0].append(t[2])
        else:
            t[0] = []
            t[0].append(t[1])

    def topology_rule_point_to_point(self, t):
        """
        topology_rule : IDENTIFIER topology_arrow IDENTIFIER SEMICOLON
                    | IDENTIFIER topology_arrow IDENTIFIER COLON topology_rule_parameters SEMICOLON
        """
        parameters = {}
        if len(t) == 7:
            parameters = t[5]
        l = TopologyRuleHost(t[1])
        r = TopologyRuleHost(t[3])
        t[0] = TopologyRule(l, t[2], r, parameters=parameters)

    def topology_rule_broadcast(self, t):
        """
        topology_rule : IDENTIFIER ARROWRIGHT STAR COLON topology_rule_parameters SEMICOLON
        """
        parameters = {}
        if len(t) == 7:
            parameters = t[5]
        l = TopologyRuleHost(t[1])
        r = TopologyRuleHost(t[3])
        t[0] = TopologyRule(l, t[2], r, parameters=parameters)

    def topology_rule_parameters(self, t):
        """
        topology_rule_parameters : topology_rule_parameter
                            | topology_rule_parameters COMMA topology_rule_parameter
        """
        t[0] = t[1]
        if len(t) == 4:
            t[0].update(t[3])

    def topology_rule_quality_parameter(self, t):
        """
        topology_rule_parameter : Q_PARAMETER EQUAL number
        """
        t[0] = {'q': t[3]}

    def topology_arrow(self, t):
        """
        topology_arrow : ARROWRIGHT
            | ARROWLEFT
            | ARROWBOTH
        """
        t[0] = t[1]

    def _extend(self):
        
        self.parser.add_state('communication', 'inclusive')

        self.parser.add_reserved_word('communication', 'COMMUNICATION_SPECIFICATION',
                                      func=self.word_communication_specification)
        self.parser.add_reserved_word('medium', 'MEDIUM_SPECIFICATION', state='communication',)
        self.parser.add_reserved_word('default_q', 'QUALITY_DEFAULT_PARAMETER', state='communication',)
        self.parser.add_reserved_word('q', 'Q_PARAMETER', state='communication',)
        self.parser.add_reserved_word('topology', 'TOPOLOGY_SPECIFICATION', state='communication',)

        self.parser.add_token('BLOCKOPEN', func=self.token_block_open, states=['communication'])
        self.parser.add_token('BLOCKCLOSE', func=self.token_block_close, states=['communication'])
        self.parser.add_token('ARROWRIGHT', r'\-\>', states=['communication'])
        self.parser.add_token('ARROWLEFT', r'\<\-', states=['communication'])
        self.parser.add_token('ARROWBOTH', r'\<\-\>', states=['communication'])

        self.parser.add_rule(self.communication_specification)
        self.parser.add_rule(self.comm_specifications)
        self.parser.add_rule(self.medium_specification)
        self.parser.add_rule(self.medium_elements)
        self.parser.add_rule(self.medium_default_parameters)
        self.parser.add_rule(self.quality_default_parameter)
        self.parser.add_rule(self.medium_topology)
        self.parser.add_rule(self.topology_rules_list)
        self.parser.add_rule(self.topology_rule_point_to_point)
        self.parser.add_rule(self.topology_rule_broadcast)
        self.parser.add_rule(self.topology_rule_parameters)
        self.parser.add_rule(self.topology_rule_quality_parameter)
        self.parser.add_rule(self.topology_arrow)


class ConfigParserExtension(LexYaccParserExtension):
    """
    Extension for parsing functions
    """
    
    def __init__(self):
        LexYaccParserExtension.__init__(self)
        self.builder = Builder()
        self.open_blocks_cnt = 0

    ##########################################
    #           RESERVED WORDS
    ##########################################
    
    def word_communication_specification(self, t):
        t.lexer.push_state('versioncommunication')
        return t
    
    ##########################################
    #                TOKENS
    ##########################################

    def token_block_open(self, t):
        r'{'
        self.open_blocks_cnt += 1
        return t

    def token_block_close(self, t):
        r'}'
        self.open_blocks_cnt -= 1
        if self.open_blocks_cnt == 0:
            t.lexer.pop_state()
        return t
    
    ##########################################
    #                RULES
    ##########################################
    
    def version_communication(self, t):
        """
        version_communication : COMMUNICATION_SPECIFICATION BLOCKOPEN version_comm_specifications BLOCKCLOSE
        """
        t[0] = {
            'mediums': t[3]
        }
    
    
    def version_comm_specifications(self, t):
        """
        version_comm_specifications : version_comm_specification
                    | version_comm_specifications version_comm_specification
        """
        t[0] = t[1]
        if len(t) == 3:
            t[0].update(t[2])

    #  Medium

    def version_medium_specification(self, t):
        """
        version_comm_specification : MEDIUM_SPECIFICATION SQLPARAN IDENTIFIER SQRPARAN BLOCKOPEN version_medium_elements BLOCKCLOSE
        """
        t[0] = {
            t[3]: t[6]
        }

    def version_medium_elements(self, t):
        """
        version_medium_elements : version_medium_default_parameters version_medium_topology
        """
        t[0] = {
            'topology': t[2],
            'default_parameters': t[1]
        }

    def version_medium_default_parameters(self, t):
        """
        version_medium_default_parameters : version_medium_default_parameter SEMICOLON
                                | version_medium_default_parameters version_medium_default_parameter SEMICOLON
        """
        t[0] = t[1]
        if len(t) == 4:
            t[0].update(t[2])

    def version_quality_default_parameter(self, t):
        """
        version_medium_default_parameter : QUALITY_DEFAULT_PARAMETER EQUAL number
        """
        t[0] = {'default_q': t[3]}

    def version_medium_topology(self, t):
        """
        version_medium_topology : TOPOLOGY_SPECIFICATION BLOCKOPEN version_topology_rules_list BLOCKCLOSE
        """
        t[0] = {'rules': t[3]}
    
    def version_topology_rules_list(self, t):
        """
        version_topology_rules_list : version_topology_rule
                        | version_topology_rules_list version_topology_rule
        """
        if len(t) == 3:
            t[0] = t[1]
            t[0].append(t[2])
        else:
            t[0] = []
            t[0].append(t[1])

    def version_topology_rule_point_to_point(self, t):
        """
        version_topology_rule : version_topology_rule_left_hosts version_topology_arrow version_topology_rule_right_hosts SEMICOLON
                    | version_topology_rule_left_hosts version_topology_arrow version_topology_rule_right_hosts COLON version_topology_rule_parameters SEMICOLON
        """
        parameters = {}
        if len(t) == 7:
            parameters = t[5]
        t[0] = TopologyRule(t[1], t[2], t[3], parameters=parameters)

    def version_topology_rule_boradcast(self, t):
        """
        version_topology_rule : version_topology_rule_left_hosts ARROWRIGHT STAR COLON version_topology_rule_parameters SEMICOLON
        """
        parameters = {}
        if len(t) == 7:
            parameters = t[5]
        t[0] = TopologyRule(t[1], t[2], None, parameters=parameters)

    def version_topology_rule_left_hosts(self, t):
        """
        version_topology_rule_left_hosts : IDENTIFIER
                        | version_topology_host_with_indicies
        """
        if isinstance(t[1], basestring):
            t[0] = TopologyRuleHost(t[1])
        else:
            t[0] = t[1]

    def version_topology_rule_right_hosts(self, t):
        """
        version_topology_rule_right_hosts : IDENTIFIER
                        | STAR
                        | version_topology_host_with_indicies
                        | version_topology_host_with_i_index
        """
        if isinstance(t[1], basestring):
            if t[1] == u"*":
                t[0] = None
            else:
                t[0] = TopologyRuleHost(t[1])
        else:
            t[0] = t[1]

    def version_topology_host_with_indicies(self, t):
        """
        version_topology_host_with_indicies : IDENTIFIER SQLPARAN INTEGER SQRPARAN
                | IDENTIFIER SQLPARAN INTEGER COLON SQRPARAN
                | IDENTIFIER SQLPARAN COLON INTEGER SQRPARAN
                | IDENTIFIER SQLPARAN INTEGER COLON INTEGER SQRPARAN
        """
        index_range = None
        if len(t) == 5:
            index_range = (t[3], t[3])
        elif len(t) == 6:
            if t[3] == ':':
                index_range = (None, t[4])
            else:
                index_range = (t[3], None)
        elif len(t) == 7:
            index_range = (t[3], t[5])
        t[0] = TopologyRuleHost(t[1], index_range=index_range)
    
    def version_topology_host_with_i_index(self, t):
        """
        version_topology_host_with_i_index : IDENTIFIER SQLPARAN I_INDEX SQRPARAN
                | IDENTIFIER SQLPARAN I_INDEX COMM_PLUS INTEGER SQRPARAN
                | IDENTIFIER SQLPARAN I_INDEX COMM_MINUS INTEGER SQRPARAN
        """
        
        i_shift = None
        if len(t) == 5:
            i_shift = 0
        elif len(t) == 7:
            if t[4] == '-':
                i_shift = - t[5]
            else:
                i_shift = t[5]
        t[0] = TopologyRuleHost(t[1], i_shift=i_shift)

    def version_topology_rule_parameters(self, t):
        """
        version_topology_rule_parameters : version_topology_rule_parameter
                            | version_topology_rule_parameters COMMA version_topology_rule_parameter
        """
        t[0] = t[1]
        if len(t) == 4:
            t[0].update(t[3])

    def version_topology_rule_quality_parameter(self, t):
        """
        version_topology_rule_parameter : Q_PARAMETER EQUAL number
        """
        t[0] = {'q': t[3]}

    def version_topology_arrow(self, t):
        """
        version_topology_arrow : ARROWRIGHT
            | ARROWLEFT
            | ARROWBOTH
        """
        t[0] = t[1]

    
    def _extend(self):
        
        self.parser.add_state('versioncommunication', 'inclusive')

        self.parser.add_reserved_word('communication', 'COMMUNICATION_SPECIFICATION',
                                      func=self.word_communication_specification)
        self.parser.add_reserved_word('medium', 'MEDIUM_SPECIFICATION', state='versioncommunication',)
        self.parser.add_reserved_word('default_q', 'QUALITY_DEFAULT_PARAMETER', state='versioncommunication',)
        self.parser.add_reserved_word('topology', 'TOPOLOGY_SPECIFICATION', state='versioncommunication',)
        self.parser.add_reserved_word('i', 'I_INDEX', state='versioncommunication')
        self.parser.add_reserved_word('q', 'Q_PARAMETER', state='versioncommunication',)

        self.parser.add_token('BLOCKOPEN', func=self.token_block_open, states=['versioncommunication'])
        self.parser.add_token('BLOCKCLOSE', func=self.token_block_close, states=['versioncommunication'])
        self.parser.add_token('ARROWRIGHT', r'\-\>', states=['versioncommunication'])
        self.parser.add_token('ARROWLEFT', r'\<\-', states=['versioncommunication'])
        self.parser.add_token('ARROWBOTH', r'\<\-\>', states=['versioncommunication'])

        self.parser.add_token('COMM_PLUS', r'\+', states=['versioncommunication'])
        self.parser.add_token('COMM_MINUS', r'\-', states=['versioncommunication'])


        self.parser.add_rule(self.version_communication)
        self.parser.add_rule(self.version_comm_specifications)
        self.parser.add_rule(self.version_medium_specification)
        self.parser.add_rule(self.version_medium_elements)
        self.parser.add_rule(self.version_medium_default_parameters)
        self.parser.add_rule(self.version_quality_default_parameter)
        self.parser.add_rule(self.version_medium_topology)
        self.parser.add_rule(self.version_topology_rules_list)
        self.parser.add_rule(self.version_topology_rule_point_to_point)
        self.parser.add_rule(self.version_topology_rule_boradcast)
        self.parser.add_rule(self.version_topology_rule_left_hosts)
        self.parser.add_rule(self.version_topology_rule_right_hosts)
        self.parser.add_rule(self.version_topology_rule_parameters)
        self.parser.add_rule(self.version_topology_rule_quality_parameter)
        self.parser.add_rule(self.version_topology_host_with_indicies)
        self.parser.add_rule(self.version_topology_host_with_i_index)
        self.parser.add_rule(self.version_topology_arrow)



    


from aqopa import module
from aqopa.module.timeanalysis.parser import ConfigParserExtension, ModelParserExtension
from aqopa.simulator.state import HOOK_TYPE_PRE_INSTRUCTION_EXECUTION,\
    HOOK_TYPE_SIMULATION_FINISHED
from .parser import MetricsParserExtension
from .hook import PreInstructionHook
from .model import TimeTrace, ChannelMessageTrace
from .gui import ModuleGui
from .console import PrintResultsHook

class Module(module.Module):
    """
    """
    
    def __init__(self):
        """ """
        self.timetraces     = {}        # Generated timetraces list for each simulator
                                        # (divided by simulators - the reason for dict)
        self.current_times  = {}        # Current times of hosts, key - host instance divided by simulators
        
        self.guis           = {}        # Divided by simulators - the reason for dict
        
        self.channel_request_times = {}  # Times when a request has been created
                                         # (divided by simulators - the reason for dict)
        self.channel_message_times = {}  # Times when a message has been sent
                                         # (divided by simulators - the reason for dict)
        self.channel_message_sending_times = {}  # The time of sending a message
                                         # (divided by simulators - the reason for dict)
        self.channel_message_traces = {}  # Time traces for communication steps
                                          # (divided by simulators - the reason for dict)

    def get_gui(self):
        if not getattr(self, '__gui', None):
            setattr(self, '__gui', ModuleGui(self))
        return getattr(self, '__gui', None)
    
    def extend_model_parser(self, parser):
        """
        Overriden
        """
        parser.add_extension(ModelParserExtension())
        return parser

    def extend_metrics_parser(self, parser):
        """
        Overriden
        """
        parser.add_extension(MetricsParserExtension())
        return parser

    def extend_config_parser(self, parser):
        """
        Overriden
        """
        parser.add_extension(ConfigParserExtension())
        return parser
    
    def _install(self, simulator):
        """
        """
        hook = PreInstructionHook(self, simulator)
        simulator.register_hook(HOOK_TYPE_PRE_INSTRUCTION_EXECUTION, hook)
        
        return simulator
    
    def install_console(self, simulator):
        """ Install module for console simulation """
        self._install(simulator)
        
        hook = PrintResultsHook(self, simulator)
        simulator.register_hook(HOOK_TYPE_SIMULATION_FINISHED, hook)
        
        return simulator
        
    def install_gui(self, simulator):
        """ Install module for gui simulation """
        self._install(simulator)
        return simulator
    
    def add_timetrace(self, simulator, host, process, instruction, expressions_details, started_at, length):
        """ """
        if simulator not in self.timetraces:
            self.timetraces[simulator] = []
        tt = self.timetraces[simulator]
        tt.append(TimeTrace(host, process, instruction, expressions_details, started_at, length))

    def get_timetraces(self, simulator):
        """ """
        if simulator not in self.timetraces:
            return []
        return self.timetraces[simulator]

    def add_channel_message_trace(self, simulator, channel, message,
                                  sender, sent_at, sending_time,
                                  receiver, started_waiting_at,
                                  started_receiving_at, receiving_time):
        if simulator not in self.channel_message_traces:
            self.channel_message_traces[simulator] = {}
        if channel not in self.channel_message_traces[simulator]:
            self.channel_message_traces[simulator][channel] = []
        cmt = self.channel_message_traces[simulator][channel]
        cmt.append(ChannelMessageTrace(channel, message,
                                       sender, sent_at, sending_time,
                                       receiver, started_waiting_at,
                                       started_receiving_at, receiving_time))

    def get_channel_message_traces(self, simulator, channel):
        """ """
        if simulator not in self.channel_message_traces:
            return []
        if channel not in self.channel_message_traces[simulator]:
            return []
        return self.channel_message_traces[simulator][channel]

    def get_all_channel_message_traces(self, simulator):
        """ """
        if simulator not in self.channel_message_traces:
            return []
        return self.channel_message_traces[simulator]
        
    def set_current_time(self, simulator, host, time):
        """ """
        if simulator not in self.current_times:
            self.current_times[simulator] = {}
        self.current_times[simulator][host] = time
        
    def get_current_time(self, simulator, host):
        """ """
        if simulator not in self.current_times:
            self.current_times[simulator] = {}
        if host not in self.current_times[simulator]:
            self.current_times[simulator][host] = 0
        return self.current_times[simulator][host]

    def add_message_sent_time(self, simulator, message, time):
        """ """
        if simulator not in self.channel_message_times:
            self.channel_message_times[simulator] = {}
        self.channel_message_times[simulator][message] = time

    def get_message_sent_time(self, simulator, message):
        if simulator not in self.channel_message_times:
            return None
        if message not in self.channel_message_times[simulator]:
            return None
        return self.channel_message_times[simulator][message]

    def add_message_sending_time(self, simulator, message, time):
        """ """
        if simulator not in self.channel_message_sending_times:
            self.channel_message_sending_times[simulator] = {}
        self.channel_message_sending_times[simulator][message] = time

    def get_message_sending_time(self, simulator, message):
        if simulator not in self.channel_message_sending_times:
            return None
        if message not in self.channel_message_sending_times[simulator]:
            return None
        return self.channel_message_sending_times[simulator][message]

    def add_request_created_time(self, simulator, request, time):
        """ """
        if simulator not in self.channel_request_times:
            self.channel_request_times[simulator] = {}
        self.channel_request_times[simulator][request] = time

    def get_request_created_time(self, simulator, request):
        if simulator not in self.channel_request_times:
            return None
        if request not in self.channel_request_times[simulator]:
            return None
        return self.channel_request_times[simulator][request]




#!/usr/bin/env python

from aqopa import module
from .gui import ModuleGui
from aqopa.simulator.state import HOOK_TYPE_SIMULATION_FINISHED
from .console import PrintResultsHook

"""
@file       __init__.py
@brief      initial file for the financialanalysis module
@author     Katarzyna Mazur
"""


class Module(module.Module):
    def __init__(self, energyanalysis_module):
        self.energyanalysis_module = energyanalysis_module
        self.consumption_costs = {}
        self.cost_per_kWh = 0

    def get_cost_per_kWh(self):
        return self.cost_per_kWh

    def set_cost_per_kWh(self, cost_per_kWh):
        self.cost_per_kWh = cost_per_kWh

    def get_gui(self):
        if not getattr(self, '__gui', None):
            setattr(self, '__gui', ModuleGui(self))
        return getattr(self, '__gui', None)

    def _install(self, simulator):
        """
        """
        return simulator

    def install_console(self, simulator):
        """ Install module for console simulation """
        self._install(simulator)
        hook = PrintResultsHook(self, simulator)
        simulator.register_hook(HOOK_TYPE_SIMULATION_FINISHED, hook)
        return simulator

    def install_gui(self, simulator):
        """ Install module for gui simulation """
        self._install(simulator)
        return simulator

    def __convert_to_joules(self, millijoules):
        return millijoules / 1000.0

    def __convert_to_kWh(self, joules):
        return joules / 3600000.0

    def calculate_cost(self, consumed_joules, cost_per_kWh):
        kWhs = self.__convert_to_kWh(consumed_joules)
        cost = kWhs * cost_per_kWh
        return cost

    def calculate_cost_for_host(self, simulator, host, cost_per_kWh):
        all_consumptions = self.get_all_hosts_consumption(simulator, simulator.context.hosts)
        joules = all_consumptions[host]['energy']
        cost_for_host = self.calculate_cost(joules, cost_per_kWh)
        return cost_for_host

    def calculate_all_costs(self, simulator, hosts, cost_per_kWh):
        all_costs = {}
        for host in hosts:
            all_costs[host] = self.calculate_cost_for_host(simulator, host, cost_per_kWh)
            self.add_cost(simulator, host, all_costs[host])
        return all_costs

    def add_cost(self, simulator, host, cost):
        """
        @brief adds cost of power consumption to
        the list of cost consumptions for the
        particular host present in the
        QoP-ML's model
        """
        # add a new simulator if not available yet
        if simulator not in self.consumption_costs:
            self.consumption_costs[simulator] = {}
        # add a new host if not available yet
        if host not in self.consumption_costs[simulator]:
            self.consumption_costs[simulator][host] = []
        # add cost for the host - but only if we
        # have not added it yet and if it is not 'empty'
        if cost not in self.consumption_costs[simulator][host] and cost:
            self.consumption_costs[simulator][host].append(cost)

    def get_min_cost(self, simulator, hosts):
        host = hosts[0]
        min_cost = self.consumption_costs[simulator][hosts[0]]
        if len(min_cost) > 0 :
            for h in hosts:
                if self.consumption_costs[simulator][h] < min_cost:
                    min_cost = self.consumption_costs[simulator][h]
                    host = h
            return min_cost[0], host
        else :
            return 0, host

    def get_max_cost(self, simulator, hosts):
        host = hosts[0]
        max_cost = self.consumption_costs[simulator][hosts[0]]
        if len(max_cost) > 0 :
            for h in hosts:
                if self.consumption_costs[simulator][h] > max_cost:
                    max_cost = self.consumption_costs[simulator][h]
                    host = h
            return max_cost[0], host
        else:
            return 0, host

    def get_avg_cost(self, simulator, hosts):
        cost_sum = 0.0
        i = 0
        for host in hosts:
            for cost in self.consumption_costs[simulator][host]:
                cost_sum += cost
                i += 1
        if i != 0:
            return cost_sum / i
        else:
            return 0

    def get_total_cost(self, simulator, hosts):
        cost_sum = 0.0
        for host in hosts:
            for cost in self.consumption_costs[simulator][host]:
                cost_sum += cost
        return cost_sum

    def get_all_costs(self, simulator):
        if simulator not in self.consumption_costs:
            return []
        return self.consumption_costs[simulator]

    # def set_all_costs(self, consumption_costs):
    # self.consumption_costs = copy.deepcopy(consumption_costs)

    def get_all_hosts_consumption(self, simulator, hosts):
        voltage = self.energyanalysis_module.get_voltage()
        consumptions = self.energyanalysis_module.get_hosts_consumptions(simulator, hosts, voltage)
        return consumptions


# !/usr/bin/env python

from aqopa import module
from .gui import ModuleGui
from aqopa.simulator.state import HOOK_TYPE_SIMULATION_FINISHED
from .console import PrintResultsHook


class Module(module.Module):
    def __init__(self, energyanalysis_module):
        self.energyanalysis_module = energyanalysis_module
        self.carbon_dioxide_emissions = {}
        self.pounds_of_co2_per_kWh = 0

    def get_pounds_of_co2_per_kWh(self):
        return self.pounds_of_co2_per_kWh

    def set_pounds_of_co2_per_kWh(self, pounds_of_co2_per_kWh):
        self.pounds_of_co2_per_kWh = pounds_of_co2_per_kWh

    def get_gui(self):
        if not getattr(self, '__gui', None):
            setattr(self, '__gui', ModuleGui(self))
        return getattr(self, '__gui', None)

    def _install(self, simulator):
        """
        """
        return simulator

    def install_console(self, simulator):
        """ Install module for console simulation """
        self._install(simulator)
        hook = PrintResultsHook(self, simulator)
        simulator.register_hook(HOOK_TYPE_SIMULATION_FINISHED, hook)
        return simulator

    def install_gui(self, simulator):
        """ Install module for gui simulation """
        self._install(simulator)
        return simulator

    def __convert_to_joules(self, millijoules):
        return millijoules / 1000.0

    def __convert_to_kWh(self, joules):
        return joules / 3600000.0

    def calculate_emission(self, consumed_joules, pounds_of_co2_per_kWh):
        kWhs = self.__convert_to_kWh(consumed_joules)
        pounds = kWhs * pounds_of_co2_per_kWh
        return pounds

    def calculate_emission_for_host(self, simulator, host, pounds_of_co2_per_kWh):
        all_consumptions = self.get_all_hosts_consumption(simulator)
        joules = all_consumptions[host]['energy']
        pounds_for_host = self.calculate_emission(joules, pounds_of_co2_per_kWh)
        return pounds_for_host

    def calculate_all_emissions(self, simulator, hosts, pounds_of_co2_per_kWh):
        all_emissions = {}
        for host in hosts:
            all_emissions[host] = self.calculate_emission_for_host(simulator, host, pounds_of_co2_per_kWh)
            self.add_co2_emission(simulator, host, all_emissions[host])
        return all_emissions

    def add_co2_emission(self, simulator, host, co2_emission):
        # add a new simulator if not available yet
        if simulator not in self.carbon_dioxide_emissions:
            self.carbon_dioxide_emissions[simulator] = {}
        # add a new host if not available yet
        if host not in self.carbon_dioxide_emissions[simulator]:
            self.carbon_dioxide_emissions[simulator][host] = []
        # add he amount of released carbon dioxide for the
        # host - but only if we have not added it yet and
        # if it is not 'empty'
        if co2_emission not in self.carbon_dioxide_emissions[simulator][host] and co2_emission:
            self.carbon_dioxide_emissions[simulator][host].append(co2_emission)

    def get_min_emission(self, simulator, hosts):
        host = hosts[0]
        min_cost = self.carbon_dioxide_emissions[simulator][hosts[0]]
        if len(min_cost) > 0:
            for h in hosts:
                if self.carbon_dioxide_emissions[simulator][h] < min_cost:
                    min_cost = self.carbon_dioxide_emissions[simulator][h]
                    host = h
            return min_cost[0], host
        else:
            return 0, None

    def get_max_emission(self, simulator, hosts):
        host = hosts[0]
        max_cost = self.carbon_dioxide_emissions[simulator][hosts[0]]
        if len(max_cost) > 0:
            for h in hosts:
                if self.carbon_dioxide_emissions[simulator][h] > max_cost:
                    max_cost = self.carbon_dioxide_emissions[simulator][h]
                    host = h
            return max_cost[0], host
        else :
            return 0, None

    def get_avg_emission(self, simulator, hosts):
        cost_sum = 0.0
        i = 0
        for host in hosts:
            for cost in self.carbon_dioxide_emissions[simulator][host]:
                cost_sum += cost
                i += 1
        if i != 0:
            return cost_sum / i
        else:
            return 0

    def get_total_emission(self, simulator, hosts):
        cost_sum = 0.0
        for host in hosts:
            for cost in self.carbon_dioxide_emissions[simulator][host]:
                cost_sum += cost
        return cost_sum

    def get_all_emissions(self, simulator):
        if simulator not in self.carbon_dioxide_emissions:
            return []
        return self.carbon_dioxide_emissions[simulator]

    def get_all_hosts_consumption(self, simulator):
        hosts = simulator.context.hosts
        voltage = self.energyanalysis_module.get_voltage()
        consumptions = self.energyanalysis_module.get_hosts_consumptions(simulator, hosts, voltage)
        return consumptions



#!/usr/bin/env python

import os
import wx
import wx.richtext
import wx.lib.newevent

"""
@file       models_lib_gui.py
@brief      GUI for the  models library window (in python's nomenclature it's a wx frame)
@author     Damian Rusinek
@date       created on 05-09-2013 by Damian Rusinek
@date       edited on 05-05-2014 by Katarzyna Mazur (visual improvements)
"""

ModelSelectedEvent, EVT_MODEL_SELECTED = wx.lib.newevent.NewEvent()

class LibraryTree(wx.TreeCtrl):
    """ """

    def __init__(self, *args, **kwargs):
        wx.TreeCtrl.__init__(self, *args, **kwargs)

        models_item = self.AddRoot(text="Models")
        models_dir = os.path.join(os.path.dirname(__file__),
                                  os.pardir,
                                  'library',
                                  'models')

        import xml.etree.ElementTree as ET

        for dir_root, dirs, files in os.walk(models_dir):
            if 'meta.xml' in files:
                item_key = os.path.basename(dir_root)

                tree = ET.parse(os.path.join(dir_root, 'meta.xml'))
                root = tree.getroot()

                name_child = root.find('name')
                if name_child is not None:
                    item = self.AppendItem(models_item, text=name_child.text)

                    author = root.find('author').text if root.find('author') is not None else ''
                    author_email = root.find('author_email').text if root.find('author_email') is not None else ''
                    description = root.find('description').text if root.find('description') is not None else ''

                    model_file = ''
                    metrics_file = ''
                    versions_file = ''
                    files = root.find('files')
                    if files is not None:
                        model_file = files.find('model').text if files.find('model') is not None else ''
                        metrics_file = files.find('metrics').text if files.find('metrics') is not None else ''
                        versions_file = files.find('versions').text if files.find('versions') is not None else ''

                    model_data = {
                        'root': dir_root,
                        'name': name_child.text,
                        'author': author,
                        'author_email': author_email,
                        'description': description,

                        'files': {
                            'model': model_file,
                            'metrics': metrics_file,
                            'versions': versions_file,
                        }
                    }
                    self.SetPyData(item, model_data)
        self.ExpandAll()

    def GetModelData(self, item):
        """ """
        if item in self.items_data:
            return self.items_data[item]
        return None


class ModelDescriptionPanel(wx.Panel):
    """
     @brief Creates panel for displaying information about the chosen model
    """

    def __init__(self, *args, **kwargs):
        """
         @brief Initializes and aligns gui elements, inits model's data
        """
        wx.Panel.__init__(self, *args, **kwargs)

        # class' data
        self.model_data = None

        # create dicts for opened files [only for
        # those from models library]
        self.filenames = {}

        # create text area (text edit, disabled, not editable) to show model's description
        __txtStyle = wx.TE_MULTILINE | wx.TE_READONLY | wx.TE_AUTO_URL
        self.modelsDescriptionText = wx.TextCtrl(self, style=__txtStyle)

        # create group boxes aka static boxes
        self.modelAboutBox = wx.StaticBox(self, label="About the model ...")
        self.modelsGeneralInfoBox = wx.StaticBox(self, label="General information")
        self.modelsDescriptionBox = wx.StaticBox(self, label="Description")
        self.modelsTreeBox = wx.StaticBox(self, label="Models library", style=wx.SUNKEN_BORDER)

        # create sizers = some kind of layout management
        modelAboutBoxSizer = wx.StaticBoxSizer(self.modelAboutBox, wx.VERTICAL)
        modelGeneralInfoBoxSizer = wx.StaticBoxSizer(self.modelsGeneralInfoBox, wx.VERTICAL)
        modelDescriptionBoxSizer = wx.StaticBoxSizer(self.modelsDescriptionBox, wx.VERTICAL)
        modelsTreeBoxSizer = wx.StaticBoxSizer(self.modelsTreeBox, wx.VERTICAL)

        # create buttons
        self.loadModelButton = wx.Button(self, label="Load model")
        self.loadModelButton.Hide()
        cancelButton = wx.Button(self, label="Close")

        # create static texts aka labels
        moduleNameLabel = wx.StaticText(self, label="Name: ")
        moduleAuthorsNameLabel = wx.StaticText(self, label="Author: ")
        moduleAuthorsEmailLabel = wx.StaticText(self, label="E-mail: ")

        # create font for static texts = same as default panel font, just bold
        defSysFont = self.GetFont()
        boldFont = wx.Font(pointSize=defSysFont.GetPointSize(),
                           family=defSysFont.GetFamily(),
                           style=defSysFont.GetStyle(),
                           weight=wx.FONTWEIGHT_BOLD)

        # create static texts will we displayed in bold
        moduleNameLabel.SetFont(boldFont)
        moduleAuthorsNameLabel.SetFont(boldFont)
        moduleAuthorsEmailLabel.SetFont(boldFont)

        # create editable static texts (labels) - will be modified
        self.moduleNameText = wx.StaticText(self)
        self.moduleAuthorsNameText = wx.StaticText(self)
        self.moduleAuthorsEmailText = wx.StaticText(self)

        # do some bindings -
        # loadModelButton loads the chosen model to the text area
        self.loadModelButton.Bind(wx.EVT_LEFT_UP, self.OnLoadModelClicked)
        # cancelButton simply closes the model's lib window (frame)
        cancelButton.Bind(wx.EVT_BUTTON, self.OnCancelClicked)

        # create'n'align models lib tree
        self.modelsTree = LibraryTree(self)
        sizer = wx.BoxSizer(wx.VERTICAL)
        sizer.Add(self.modelsTree, 1, wx.ALL | wx.EXPAND)
        modelsTreeBoxSizer.Add(sizer, 1, wx.ALL | wx.EXPAND, 5)

        # add some vertical space to make gui more readable
        __verticalSpace = 25

        # align name label and name
        sizer = wx.BoxSizer(wx.HORIZONTAL)
        sizer.Add(moduleNameLabel, 0, wx.ALL | wx.LEFT, 5)
        sizer.Add(wx.StaticText(self), 0, wx.ALL | wx.EXPAND, 5)
        sizer.Add(self.moduleNameText, 1, wx.ALL | wx.ALIGN_CENTRE_HORIZONTAL, 5)
        modelGeneralInfoBoxSizer.Add(sizer, 0, wx.ALL | wx.EXPAND, 5)

        # align author label and author
        sizer = wx.BoxSizer(wx.HORIZONTAL)
        sizer.Add(moduleAuthorsNameLabel, 0, wx.ALL | wx.LEFT, 5)
        sizer.Add(wx.StaticText(self), 0, wx.ALL | wx.EXPAND, 5)
        sizer.Add(self.moduleAuthorsNameText, 1, wx.ALL | wx.ALIGN_CENTRE_HORIZONTAL, 5)
        modelGeneralInfoBoxSizer.Add(sizer, 0, wx.ALL | wx.EXPAND, 5)

        # align email label and email
        sizer = wx.BoxSizer(wx.HORIZONTAL)
        sizer.Add(moduleAuthorsEmailLabel, 0, wx.ALL | wx.LEFT, 5)
        sizer.Add(wx.StaticText(self), 0, wx.ALL | wx.EXPAND, 5)
        sizer.Add(self.moduleAuthorsEmailText, 1, wx.ALL | wx.ALIGN_CENTRE_HORIZONTAL, 5)
        modelGeneralInfoBoxSizer.Add(sizer, 0, wx.ALL | wx.EXPAND, 5)

        # add group box 'general model info' at the top of the 'model's about' group box
        modelAboutBoxSizer.Add(modelGeneralInfoBoxSizer, 0, wx.ALL | wx.TOP | wx.EXPAND, 10)

        # add text area to the 'model's description' group box
        modelDescriptionBoxSizer.Add(self.modelsDescriptionText, 1, wx.EXPAND | wx.TOP | wx.ALL, 5)

        # add 'model's description' group box to the 'model's about' group box
        modelAboutBoxSizer.Add(modelDescriptionBoxSizer, 1, wx.ALL | wx.TOP | wx.EXPAND, 10)

        # align buttons
        sizer = wx.BoxSizer(wx.HORIZONTAL)
        sizer.Add(self.loadModelButton, 0, wx.LEFT | wx.ALL, 5)
        sizer.Add(cancelButton, 0, wx.LEFT | wx.ALL, 5)
        # add buttons to 'about the model' group box
        modelAboutBoxSizer.Add(sizer, 0, wx.ALIGN_RIGHT | wx.RIGHT, 10)

        # align all the gui elements together on the panel
        sizer = wx.BoxSizer(wx.HORIZONTAL)
        sizer.Add(modelsTreeBoxSizer, 1, wx.ALL | wx.EXPAND)  # or simply sizer.Add(modelsTreeBoxSizer, 0, wx.EXPAND)
        sizer.Add(modelAboutBoxSizer, 1, wx.EXPAND)

        # do the final alignment
        self.SetSizer(sizer)
        self.Layout()

    def OnCancelClicked(self, event):
        """
	     @brief     closes the frame (as well as the panel)
	    """
        frame = self.GetParent()
        frame.Close()

    def __CreateLibraryPath(self, filename):
        """
        @brief      creates a filepath displayed on GUI,
                    the filepath is the path of the
                    model/metric/version chosen from
                    AQoPA's model library
        """
        __mainPath = os.path.split(self.model_data['root'])[0]
        for i in range (0,4):
            __mainPath = os.path.split(__mainPath)[0]
        __mainPath += "/library/models/"
        __chosenModelPath = os.path.split(self.model_data['root'])[1]
        __mainPath += __chosenModelPath
        __mainPath += "/"
        __mainPath += filename
        return __mainPath

    def ShowModel(self, model_data):
        """ """
        self.model_data = model_data
        self.moduleNameText.SetLabel(model_data['name'])
        self.moduleAuthorsNameText.SetLabel(model_data['author'])
        self.moduleAuthorsEmailText.SetLabel(model_data['author_email'])
        self.modelsDescriptionText.SetValue(model_data['description'])
        self.filenames['model'] = self.__CreateLibraryPath(os.path.split(self.model_data['files']['model'])[1])
        self.filenames['metrics'] = self.__CreateLibraryPath(os.path.split(self.model_data['files']['metrics'])[1])
        self.filenames['versions'] = self.__CreateLibraryPath(os.path.split(self.model_data['files']['versions'])[1])
        self.modelsDescriptionText.Show()
        self.loadModelButton.Show()
        self.Layout()

    def OnLoadModelClicked(self, event=None):
        """ """
        f = open(os.path.join(self.model_data['root'], self.model_data['files']['model']))
        model_data = f.read()
        f.close()

        f = open(os.path.join(self.model_data['root'], self.model_data['files']['metrics']))
        metrics_data = f.read()
        f.close()

        f = open(os.path.join(self.model_data['root'], self.model_data['files']['versions']))
        versions_data = f.read()
        f.close()

        evt = ModelSelectedEvent(model_data=model_data,
                                 metrics_data=metrics_data,
                                 versions_data=versions_data,
                                 filenames=self.filenames)
        wx.PostEvent(self, evt)


class LibraryFrame(wx.Frame):
    """ """

    def __init__(self, *args, **kwargs):
        wx.Frame.__init__(self, *args, **kwargs)

        ###################
        # SIZERS & EVENTS
        ###################

        self.modelDescriptionPanel = ModelDescriptionPanel(self)
        self.modelsTree = self.modelDescriptionPanel.modelsTree
        self.modelDescriptionPanel.Bind(EVT_MODEL_SELECTED, self.OnLoadModelSelected)

        # fill panel with linear gradient to make it look fancy = NOT NOW!
        #self.modelDescriptionPanel.Bind(wx.EVT_PAINT, self.OnPaintPrettyPanel)

        self.modelsTree.Bind(wx.EVT_TREE_SEL_CHANGED, self.OnModelSelected)
        self.modelsTree.Bind(wx.EVT_TREE_ITEM_ACTIVATED, self.OnModelDoubleClicked)

        # set window's icon
        self.SetIcon(wx.Icon(self.CreatePath4Resource('models_lib.ico'), wx.BITMAP_TYPE_ICO))

        # set minimum windows' size - you can make it bigger, but not smaller!
        self.SetMinSize(wx.Size(800, 450))
        # do the final alignment
        sizer = wx.BoxSizer(wx.HORIZONTAL)
        sizer.Add(self.modelDescriptionPanel, 4, wx.EXPAND)
        self.SetSizer(sizer)
        # center model's lib window on a screen
        self.CentreOnParent()
        self.Layout()

    def CreatePath4Resource(self, resourceName):
        """
        @brief      creates and returns path to the
                    given file in the resource
                    ('assets') dir
        @return     path to the resource
        """
        tmp = os.path.split(os.path.dirname(__file__))
        return os.path.join(tmp[0], 'bin', 'assets', resourceName)

    def GetFilenames(self):
        return self.modelDescriptionPanel.filenames

    def OnPaintPrettyPanel(self, event):
        # establish the painting canvas
        dc = wx.PaintDC(self.modelDescriptionPanel)
        x = 0
        y = 0
        w, h = self.GetSize()
        dc.GradientFillLinear((x, y, w, h), '#606060', '#E0E0E0', nDirection=wx.NORTH)

    def OnModelSelected(self, event=None):
        """ """
        itemID = event.GetItem()
        if not itemID.IsOk():
            itemID = self.tree.GetSelection()
        model_data = self.modelsTree.GetPyData(itemID)
        if model_data:
            self.modelDescriptionPanel.ShowModel(model_data)

    def OnModelDoubleClicked(self, event=None):
        """ """
        itemID = event.GetItem()
        if not itemID.IsOk():
            itemID = self.tree.GetSelection()
        model_info = self.modelsTree.GetPyData(itemID)
        if model_info:
            f = open(os.path.join(model_info['root'], model_info['files']['model']))
            model_data = f.read()
            f.close()

            f = open(os.path.join(model_info['root'], model_info['files']['metrics']))
            metrics_data = f.read()
            f.close()

            f = open(os.path.join(model_info['root'], model_info['files']['versions']))
            versions_data = f.read()
            f.close()

            evt = ModelSelectedEvent(model_data=model_data,
                                     metrics_data=metrics_data,
                                     versions_data=versions_data,
                                     filenames=self.GetFilenames())
            wx.PostEvent(self, evt)
            self.Close()

    def OnLoadModelSelected(self, event=None):
        """ """
        wx.PostEvent(self, event)
        self.Close()



#!/usr/bin/env python
  
import Structs
import pickle
import wx
  

#Read from and write to a file
  
def readFile(filename):
    filename = "files\\"+filename
    f = open(filename)
    lines = [line.strip() for line in f]
    f.close()
    return lines

  
#First clears then writes
def writeFile(filename, thelist):
    filename = "files\\"+filename
    f = open(filename, 'w')
    for item in thelist:
        f.write("%s\n" % item)
    f.close()
  
  
#picklingMethods
def saveCategories(path):
    with open(path, 'wb') as handle:
        pickle.dump(Structs.categoryList, handle)
  
def loadCategories(path):
    with open(path, 'rb') as handle:
        Structs.categoryList = pickle.load(handle) 
        
def saveFacts(path):
    with open(path, 'wb') as handle:
        pickle.dump(Structs.factList, handle)
  
def loadFacts(path):
    with open(path, 'rb') as handle:
        Structs.factList = pickle.load(handle)
        
def saveSA(path):
    with open(path, 'wb') as handle:
        pickle.dump(Structs.saList, handle)
  
def loadSA(path):
    with open(path, 'rb') as handle:
        Structs.saList = pickle.load(handle)
        
def saveRules(path):
    with open(path, 'wb') as handle:
        pickle.dump(Structs.ruleList, handle)
  
def loadRules(path):
    with open(path, 'rb') as handle:
        Structs.ruleList = pickle.load(handle)
        
def saveFOs(path):
    with open(path, 'wb') as handle:
        pickle.dump(Structs.foList, handle)
  
def loadFOs(path):
    with open(path, 'rb') as handle:
        Structs.foList = pickle.load(handle)
        
def saveERs(path):
    with open(path, 'wb') as handle:
        pickle.dump(Structs.erList, handle)
  
def loadERs(path):
    with open(path, 'rb') as handle:
        Structs.erList = pickle.load(handle)
  
def saveCases(path):
    with open(path, 'wb') as handle:
        pickle.dump(Structs.caseList, handle)
  
def loadCases(path):
    with open(path, 'rb') as handle:
        Structs.caseList = pickle.load(handle) 
        
def saveEvaluations(path):
    with open(path, 'wb') as handle:
        pickle.dump(Structs.evalList, handle)
  
def loadEvaluations(path):
    with open(path, 'rb') as handle:
        Structs.evalList = pickle.load(handle) 
        


#!/usr/bin/env python

import os
import re
import wx
import wx.animate
import wx.lib.scrolledpanel as scrolled
import wx.lib.delayedresult

from aqopa.model import name_indexes
from aqopa.bin import gui as aqopa_gui
from aqopa.simulator.error import RuntimeException
from aqopa.gui.general_purpose_frame_gui import GeneralFrame

"""
@file       gui.py
@brief      GUI for the reputation analysis panel
@author     Damian Rusinek <damian.rusinek@gmail.com>
@date       created on 06-09-2013 by Damian Rusinek
@date       edited on 27-06-2014 by Katarzyna Mazur (visual improvements mainly)
"""

class SingleVersionPanel(wx.Panel):
    """ 
    Frame presenting results for one simulation.  
    Simulator may be retrived from module, 
    because each module has its own simulator.
    """
    
    def __init__(self, module, *args, **kwargs):
        wx.Panel.__init__(self, *args, **kwargs)

        self.module = module
        self.versionSimulator = {}

        self.hostChoosePanels       = []  # Panels used to choose hosts for energy consumptions results
        self.checkBoxInformations   = []  # Tuples with host name, and index ranges widget
        self.hostCheckBoxes         = []  # List of checkboxes with hosts names used for hosts' selection

        #################
        # VERSION BOX
        #################
        
        versionBox = wx.StaticBox(self, label="Version")
        versionsLabel = wx.StaticText(self, label="Choose Version To See\nAnalysis Results:")
        self.versionsList = wx.ComboBox(self, style=wx.TE_READONLY)
        self.versionsList.Bind(wx.EVT_COMBOBOX, self.OnVersionChanged)
        versionBoxSizer = wx.StaticBoxSizer(versionBox, wx.HORIZONTAL)
        versionBoxSizer.Add(versionsLabel, 0, wx.ALL | wx.ALIGN_CENTER, 5)
        versionBoxSizer.Add(wx.StaticText(self), 1, wx.ALL | wx.EXPAND, 5)
        versionBoxSizer.Add(self.versionsList, 1, wx.ALL | wx.ALIGN_CENTER, 5)

        ##################################
        # ENERGY CONSUMPTION BOX
        ##################################
        
        self.consumptionsBox = wx.StaticBox(self, label="Reputation results")

        hostsBox, hostsBoxSizer = self._BuildHostsBoxAndSizer()

        reputationBoxSizer = wx.StaticBoxSizer(self.consumptionsBox, wx.VERTICAL)
        reputationHBoxSizer = wx.BoxSizer(wx.HORIZONTAL)
        reputationHBoxSizer.Add(hostsBoxSizer, 1, wx.ALL | wx.EXPAND)
        
        self.showReputationBtn = wx.Button(self, label="Show")
        self.showReputationBtn.Bind(wx.EVT_BUTTON, self.OnShowReputationButtonClicked)
        
        reputationBoxSizer.Add(reputationHBoxSizer, 0, wx.ALL | wx.EXPAND)

        buttonsBoxSizer = wx.BoxSizer(wx.HORIZONTAL)
        buttonsBoxSizer.Add(wx.StaticText(self), 1, wx.EXPAND, 5)
        buttonsBoxSizer.Add(self.showReputationBtn, 0, wx.ALL | wx.ALIGN_RIGHT)

        reputationBoxSizer.Add(buttonsBoxSizer, 1, wx.ALL | wx.EXPAND, 5)
        
        sizer = wx.BoxSizer(wx.VERTICAL)
        sizer.Add(versionBoxSizer, 0, wx.ALL | wx.EXPAND, 5)
        sizer.Add(reputationBoxSizer, 1, wx.ALL | wx.EXPAND, 5)
        self.SetSizer(sizer)
        
        self.SetVersionsResultsVisibility(False)
        
    #################
    # REACTIONS
    #################
        
    def AddFinishedSimulation(self, simulator):
        """ """
        version = simulator.context.version
        self.versionsList.Append(version.name)
        
        self.versionSimulator[version.name] = simulator
        
    def OnVersionChanged(self, event):
        """ """
        versionName = self.versionsList.GetValue()
        simulator = self.versionSimulator[versionName]
        
        self._BuildHostsChoosePanel(simulator)
        self.SetVersionsResultsVisibility(True)
        
    def OnShowReputationButtonClicked(self, event):
        """ """
        versionName = self.versionsList.GetValue()
        simulator = self.versionSimulator[versionName]
        hosts = self._GetSelectedHosts(simulator)
        
        if len(hosts) == 0:
            wx.MessageBox("Please select hosts.", 'Error', wx.OK | wx.ICON_ERROR)
            return

        self.ShowHostsReputation(simulator, hosts)

    def RemoveAllSimulations(self):
        """ """
        self.versionsList.Clear()
        self.versionsList.SetValue("")
        self.versionSimulator = {}

        self.hostChoosePanels = []
        self.checkBoxInformations = {}
        self.hostCheckBoxes = []
        self.hostsBoxSizer.Clear(True)

        self.SetVersionsResultsVisibility(False)

    #################
    # LAYOUT
    #################
    
    def _BuildHostsBoxAndSizer(self):
        """ """
        self.hostsBox = wx.StaticBox(self, label="Host(s)")
        self.hostsBoxSizer = wx.StaticBoxSizer(self.hostsBox, wx.VERTICAL)
        return self.hostsBox, self.hostsBoxSizer
    
    def _BuildHostsChoosePanel(self, simulator):
        """ """
        for p in self.hostChoosePanels:
            p.Destroy()
        self.hostChoosePanels = []
        self.checkBoxInformations = {}
        self.hostCheckBoxes = []
        
        self.hostsBoxSizer.Layout()
        
        hosts = simulator.context.hosts
        hostsIndexes = {} 
        for h in hosts:
            name = h.original_name()
            indexes = name_indexes(h.name)
            index = indexes[0]
            
            if name not in hostsIndexes or index > hostsIndexes[name]:
                hostsIndexes[name] = index
                
        for hostName in hostsIndexes:
            
            panel = wx.Panel(self)
            panelSizer = wx.BoxSizer(wx.HORIZONTAL)
            
            ch = wx.CheckBox(panel, label=hostName, size=(120, 20))
            textCtrl = wx.TextCtrl(panel, size=(200, 20))
            textCtrl.SetValue("0")
            
            rangeLabel = "Available range: 0"
            if hostsIndexes[hostName] > 0:
                rangeLabel += " - %d" % hostsIndexes[hostName] 
            maxLbl = wx.StaticText(panel, label=rangeLabel)
            
            panelSizer.Add(ch, 0, wx.ALL | wx.ALIGN_CENTER)
            panelSizer.Add(wx.StaticText(self), 1, wx.ALL | wx.ALIGN_RIGHT)
            panelSizer.Add(textCtrl, 0, wx.ALL | wx.ALIGN_CENTER)
            panelSizer.Add(maxLbl, 0, wx.ALL | wx.ALIGN_CENTER)
            panel.SetSizer(panelSizer)
            self.hostsBoxSizer.Add(panel, 1, wx.ALL)
            
            self.checkBoxInformations[ch] = (hostName, textCtrl)
            self.hostChoosePanels.append(panel)
            self.hostCheckBoxes.append(ch)
            
        self.hostsBoxSizer.Layout()
        self.Layout()

    def SetVersionsResultsVisibility(self, visible):
        """ """
        widgets = []
        widgets.append(self.consumptionsBox)
        widgets.append(self.hostsBox)
        widgets.append(self.showReputationBtn)

        for w in widgets:
            if visible:
                w.Show()
            else:
                w.Hide()
                
        self.Layout()
    
    #################
    # STATISTICS
    #################
        
    def _GetSelectedHosts(self, simulator):
        """ Returns list of hosts selected by user """
        
        def ValidateHostsRange(indexesRange):
            """ """
            return re.match(r'\d(-\d)?(,\d(-\d)?)*', indexesRange)
        
        def GetIndexesFromRange(indexesRange):
            """ Extracts numbers list of hosts from range text """
            indexes = []
            ranges = indexesRange.split(',')
            for r in ranges:
                parts = r.split('-')
                if len(parts) == 1:
                    indexes.append(int(parts[0]))
                else:
                    for i in range(int(parts[0]), int(parts[1])+1):
                        indexes.append(i)
            return indexes
        
        hosts = []
        for ch in self.hostCheckBoxes:
            if not ch.IsChecked():
                continue
            hostName, hostRangeTextCtrl = self.checkBoxInformations[ch]
            indexesRange = hostRangeTextCtrl.GetValue()
            if not ValidateHostsRange(indexesRange):
                wx.MessageBox("Range '%s' for host '%s' is invalid. Valid example: 0,12,20-25,30." 
                              % (indexesRange, hostName), 'Error', wx.OK | wx.ICON_ERROR)
                break
            else:
                indexes = GetIndexesFromRange(indexesRange)
                for h in simulator.context.hosts:
                    hostIndexes = name_indexes(h.name)
                    if h.original_name() == hostName and hostIndexes[0] in indexes:
                        hosts.append(h)
        return hosts
        
    
    def ShowHostsReputation(self, simulator, hosts):
        """ """
        lblText = ""

        for h in hosts:
            lblText += "%s: " % (h.name,)
            error = h.get_finish_error()
            if error is not None:
                lblText += " (Not Finished - %s)" % error

            host_vars = self.module.get_host_vars(h)
            for var_name in host_vars:
                lblText = "%s: %s" % (var_name, unicode(host_vars[var_name]))
            lblText += "\n\n"

        # create a new frame to show time analysis results on it
        hostsReputationWindow = GeneralFrame(self, "Reputation Analysis Results", "Host's Reputation", "modules_results.png")

        # create scrollable panel
        hostsPanel = scrolled.ScrolledPanel(hostsReputationWindow)

        # create informational label
        lbl = wx.StaticText(hostsPanel, label=lblText)

        # sizer to align gui elements properly
        sizer = wx.BoxSizer(wx.VERTICAL)
        sizer.Add(lbl, 0, wx.ALL | wx.EXPAND, 5)
        hostsPanel.SetSizer(sizer)
        hostsPanel.SetupScrolling(scroll_x=True)
        hostsPanel.Layout()

        # add panel on a window
        hostsReputationWindow.AddPanel(hostsPanel)
        # center window on a screen
        hostsReputationWindow.CentreOnScreen()
        # show the results on the new window
        hostsReputationWindow.Show()
    
class MainResultsNotebook(wx.Notebook):
    """ """
    def __init__(self, module, *args, **kwargs):
        wx.Notebook.__init__(self, *args, **kwargs)

        il = wx.ImageList(24, 24)
        singleVersionImg = il.Add(wx.Bitmap(self.CreatePath4Resource('reputation.png'), wx.BITMAP_TYPE_PNG))
        self.AssignImageList(il)

        self.module = module
        
        self.oneVersionTab = SingleVersionPanel(self.module, self)
        self.AddPage(self.oneVersionTab, "Single Version")
        self.SetPageImage(0, singleVersionImg)
        self.oneVersionTab.Layout()
        
    def OnParsedModel(self):
        """ """
        self.oneVersionTab.RemoveAllSimulations()
        
    def OnSimulationFinished(self, simulator):
        """ """
        self.oneVersionTab.AddFinishedSimulation(simulator)
        
    def OnAllSimulationsFinished(self, simulators):
        """ """
        pass

    def CreatePath4Resource(self, resourceName):
        """
        @brief      creates and returns path to the
                    given file in the resource
                    ('assets') dir
        @return     path to the resource
        """
        tmp = os.path.split(os.path.dirname(__file__))
        # find last / character in path
        idx = tmp[0].rfind('/')
        # get substring - path for resource
        path = tmp[0][0:idx]
        return os.path.join(path, 'bin', 'assets', resourceName)
        
class ModuleGui(wx.EvtHandler):
    """
    Class used by GUI version of AQoPA.
    """
    
    def __init__(self, module):
        """ """
        wx.EvtHandler.__init__(self)
        
        self.module = module
        self.mainResultNotebook = None
        
    def get_name(self):
        return "Reputation"
    
    def get_configuration_panel(self, parent):
        """ Returns WX panel with configuration controls. """
        
        panel = wx.Panel(parent)
        
        sizer = wx.BoxSizer(wx.VERTICAL)
        text = wx.StaticText(panel, label="Module does not need to be configured.") 
        sizer.Add(text, 0, wx.ALL | wx.EXPAND, 5)
        text = wx.StaticText(panel, label="All result options will be available after results are calculated.")
        sizer.Add(text, 0, wx.ALL | wx.EXPAND, 5)
        
        panel.SetSizer(sizer)
        return panel
    
    def get_results_panel(self, parent):
        """
        Create main result panel existing from the beginning 
        which will be extended when versions' simulations are finished.
        """
        self.mainResultNotebook = MainResultsNotebook(self.module, parent)
        return self.mainResultNotebook
    
    def on_finished_simulation(self, simulator):
        """ """
        self.mainResultNotebook.OnSimulationFinished(simulator)
    
    def on_finished_all_simulations(self, simulators):
        """ 
        Called once for all simulations after all of them are finished.
        """
        self.mainResultNotebook.OnAllSimulationsFinished(simulators)
        
    def on_parsed_model(self):
        """ """
        self.mainResultNotebook.OnParsedModel()
        
        

        
        
        
        



'''
Created on 31-05-2013

@author: Damian Rusinek <damian.rusinek@gmail.com>
'''
from math import ceil
import random
from aqopa.simulator.state import Hook, ExecutionResult
from aqopa.model import AssignmentInstruction,\
    CallFunctionInstruction, IfInstruction, WhileInstruction,\
    CommunicationInstruction, CallFunctionExpression, TupleExpression, ComparisonExpression, COMMUNICATION_TYPE_OUT, \
    COMMUNICATION_TYPE_IN
from aqopa.module.timeanalysis.error import TimeSynchronizationException
from aqopa.simulator.error import RuntimeException

class PreInstructionHook(Hook):
    """
    Execution hook executed before default core execution of each instruction.
    Returns execution result.
    """
    
    def __init__(self, module, simulator):
        """ """
        self.module = module
        self.simulator = simulator

    def execute(self, context, **kwargs):
        """
        """
        instruction = context.get_current_instruction()
        
        if instruction.__class__ not in [AssignmentInstruction, CallFunctionInstruction, 
                                     IfInstruction, WhileInstruction, CommunicationInstruction]:
            return
        
        if isinstance(instruction, CommunicationInstruction):
            return self._execute_communication_instruction(context, **kwargs)
        else:
            self._update_time(context)
            return ExecutionResult()
            
    def _update_time(self, context):
        """ 
        Update times in context according to current instruction.
        """
        instruction = context.get_current_instruction()
        
        if isinstance(instruction, AssignmentInstruction):
            expression = instruction.expression
        elif isinstance(instruction, CallFunctionInstruction):
            expression = CallFunctionExpression(instruction.function_name, 
                                                instruction.arguments, 
                                                instruction.qop_arguments)
        else:
            expression = instruction.condition
            
        # Return details for each expression in instruction
        # In some instruction may be more expressions (tuple, nested call function)
        total_time, time_details = self._get_time_details_for_expression(context, expression)

        if total_time > 0:
            host = context.get_current_host()
            self.module.add_timetrace(self.simulator, host, host.get_current_process(),
                                      instruction, time_details,
                                      self.module.get_current_time(self.simulator, host),
                                      total_time)
            self.module.set_current_time(self.simulator, host, self.module.get_current_time(self.simulator, host) + total_time)
            
    def _get_time_details_for_expression(self, context, expression):
        """
        Returns calculated time that execution of expression takes.
        """
        if isinstance(expression, TupleExpression):
            return self._get_time_details_for_tuple_expression(context, expression)
        elif isinstance(expression, CallFunctionExpression):
            return self._get_time_details_for_simple_expression(context, expression)
        elif isinstance(expression, ComparisonExpression):
            return self._get_time_details_for_comparison_expression(context, expression)
        return 0, []

    def _get_time_details_for_tuple_expression(self, context, expression):
        """
        Calculate execution time for tuple expression. 
        """
        total_time = 0
        time_details = [] 
        for i in range(0, len(expression.elements)):
            element_total_time, element_time_details = self._get_time_details_for_expression(context, expression.elements[i])
            total_time += element_total_time
            time_details.extend(element_time_details)
        return total_time, time_details

    def _get_time_details_for_comparison_expression(self, context, expression):
        """
        Calculate execution time for tuple expression.
        """
        total_time = 0
        time_details = []

        element_total_time, element_time_details = self._get_time_details_for_expression(context, expression.left)
        total_time += element_total_time
        time_details.extend(element_time_details)

        element_total_time, element_time_details = self._get_time_details_for_expression(context, expression.right)
        total_time += element_total_time
        time_details.extend(element_time_details)

        return total_time, time_details
    
    def _get_time_for_exact_range_metric(self, metric_type, metric_unit, metric_value, expression, host, context):
        """
        Returns execution time of expression which has exact or range metric assigned.
        """

        if metric_unit in ["ms", "s"]:
            # Get time
            if metric_type == "exact":
                time_value = float(metric_value)
            else:  # range
                mvalues = metric_value.split('..')
                val_from = float(mvalues[0])
                val_to = float(mvalues[1])
                time_value = val_from + (val_to-val_from)*random.random()

            # Update time according to the time unit
            if metric_unit == "ms":
                return time_value / 1000.0
            else:  # metric_unit == "s":
                return time_value

        elif metric_unit in ["mspb", "mspB", "kbps", "mbps"]:

            mparts = metric_value.split(':')
            time_metric = mparts[0]
            indexes = None 
            if len(mparts) == 2:
                indexes = [int(i) for i in mparts[1].split(',')]

            # Get time
            if metric_type == "exact":
                time_value = float(time_metric)
            else:  # range
                mvalues = time_metric.split('..')
                val_from = float(mvalues[0])
                val_to = float(mvalues[1])
                time_value = val_from + (val_to-val_from)*random.random()

            size = 0
            if indexes is None:
                populated_expression = context.expression_populator.populate(expression, 
                                                                             context.get_current_host())
                size = context.metrics_manager.get_expression_size(populated_expression, 
                                                                   context, context.get_current_host())
            else:
                for index in indexes:
                    populated_expression = context.expression_populator.populate(
                        expression.arguments[index-1], context.get_current_host())

                    size += context.metrics_manager.get_expression_size(
                        populated_expression, context, context.get_current_host())

            if metric_unit in ["mspb", "mspB"]:
                msperbyte = float(time_value)
                if metric_unit == "mspb":
                    msperbyte /= 8.0
            elif metric_unit == "kbps":
                msperbyte = 8000.0 / 1024.0 / time_value
            else: # mbps
                msperbyte = 8000.0 / 1024.0 / 1024.0 / time_value

            # Make s from ms
            sperbyte = msperbyte / 1000.0
            return sperbyte * size
    
    def _get_time_for_block_metric(self, metric_type, metric_unit, metric_value, expression, host, context):
        """
        Returns execution time of expression which has block metric assigned.
        """
        mparts = metric_value.split(':')
        element_index = None
        if len(mparts) == 3:
            element_index = int(mparts[0])-1
            unit_time = float(mparts[1])
            unit_size = int(mparts[2])
        else:
            unit_time = float(mparts[0])
            unit_size = int(mparts[1])

        time_unit = metric_unit[0]
        if time_unit == "ms":
            unit_time /= 1000.0
        size_unit = metric_unit[1]

        expression_to_populate = expression if element_index is None else expression.arguments[element_index]
        populated_expression = context.expression_populator.populate(expression_to_populate, host)
        argument_size = context.metrics_manager.get_expression_size(populated_expression,
                                                                    context, host)
        units = ceil(argument_size / float(unit_size))

        time = units * unit_time
        if size_unit == 'b':
            time *= 8.0
        return time
    
    def _get_time_details_for_simple_expression(self, context, expression):
        """
        Calculate time for expression.
        """
        time = 0
        time_details = []
        
        metric = context.metrics_manager\
            .find_primitive(context.get_current_host(), expression)

        if metric:
            block = metric.block
            
            for i in range(0, len(block.service_params)):
                sparam = block.service_params[i]
                
                if sparam.service_name.lower().strip() != "time":
                    continue

                metric_type = sparam.param_name.lower().strip()
                metric_unit = sparam.unit
                metric_value = metric.service_arguments[i].strip()

                if metric_type in ["exact", "range"]:

                    time = self._get_time_for_exact_range_metric(metric_type, metric_unit, metric_value, 
                                                                 expression, context.get_current_host(), context)
                elif metric_type == "block":

                    time = self._get_time_for_block_metric(metric_type, metric_unit, metric_value,
                                                           expression, context.get_current_host(), context)
                elif metric_type == 'algorithm':
                    algorithm_name = metric_value
                    if not context.algorithms_resolver.has_algorithm(algorithm_name):
                        raise RuntimeException("Communication algorithm {0} undeclared.".format(algorithm_name))

                    alg = context.algorithms_resolver.get_algorithm(algorithm_name)
                    variables = {alg['parameter']: expression}
                    time = context.algorithms_resolver.calculate(context, context.get_current_host(),
                                                                 algorithm_name, variables)

                    # Update time according to the time unit
                    if metric_unit == "ms":
                        time /= 1000.0

            if time > 0:
                time_details.append((expression, time))

        for expr in expression.arguments:
            argument_time, argument_time_details = self._get_time_details_for_expression(context, expr)
            time += argument_time
            time_details.extend(argument_time_details)

        return time, time_details

    def _get_time_for_communication_step(self, context, host, channel, metric, message, receiver=None):
        """ Returns time of sending/receiving message """

        if metric['type'] == 'metric':
            metric_value = metric['value']
        elif metric['type'] == 'algorithm':
            algorithm_name = metric['name']
            if not context.algorithms_resolver.has_algorithm(algorithm_name):
                raise RuntimeException("Communication algorithm {0} undeclared.".format(algorithm_name))

            link_quality = context.channels_manager.get_router().get_link_quality(channel.tag_name,
                                                                                  message.sender,
                                                                                  receiver)
            alg = context.algorithms_resolver.get_algorithm(algorithm_name)
            variables = {
                'link_quality': link_quality,
                alg['parameter']: message.expression,
            }
            metric_value = context.algorithms_resolver.calculate(context, host, algorithm_name, variables)
        else:
            return 0

        unit = metric['unit']
        if unit == 'ms':
            # exact time
            return metric_value / 1000.0
        else:
            # time depending on the size
            populated_expression = context.expression_populator.populate(message.expression, message.sender)
            # size in bytes
            size = context.metrics_manager.get_expression_size(populated_expression, context, message.sender)

            mspB = 0
            if unit == 'mspB':
                mspB = float(metric_value)
            elif unit == 'mspb':
                mspB = float(metric_value) * 8.0
            elif unit == 'kbps':
                mspB = 1.0 / (float(metric_value) * 0.128)
            elif unit == 'mbps':
                mspB = 1.0 / (float(metric_value) * 0.000128)

            spB = mspB / 1000.0

            return spB * size

    def _find_communication_metric(self, context, channel, sender, receiver=None):
        """
        Return the communication metric from topology.
        """
        return context.channels_manager.get_router().get_link_parameter_value('time', channel.tag_name, sender, receiver)

    def _get_time_of_sending_point_to_point(self, context, channel, message, request):
        """ Returns time of message sending process """
        metric = self._find_communication_metric(context, channel, message.sender, receiver=request.receiver)
        if metric is None:
            return 0
        return self._get_time_for_communication_step(context, message.sender, channel, metric, message, receiver=request.receiver)

    def _get_time_of_sending_boradcast(self, context, channel, message):
        """ Returns time of message sending process """
        metric = self._find_communication_metric(context, channel, message.sender, receiver=None)
        if metric is None:
            return 0
        return self._get_time_for_communication_step(context, message.sender, channel, metric, message, receiver=None)

    def _get_time_of_receiving(self, context, channel, message, request):
        """ Returns time of message receiving process """
        metric = self._find_communication_metric(context, channel, message.sender, receiver=request.receiver)
        if metric is None:
            return 0
        return self._get_time_for_communication_step(context, message.sender, channel, metric, message, receiver=request.receiver)

    def _execute_communication_instruction(self, context, **kwargs):
        """ """
        channel = context.channels_manager.find_channel_for_current_instruction(context)
        if not channel:
            return ExecutionResult(result_kwargs=kwargs)

        if kwargs is None:
            kwargs = {}

        instruction = context.get_current_instruction()

        # TODO: Consider situation when host has many context and in one context it is waiting for message
        # while in second one it can go further (non communication instruction)

        # Check if other hosts should send or receive their message through this channel before
        current_host_time = self.module.get_current_time(self.simulator, context.get_current_host())
        min_hosts_time = (context.get_current_host(), current_host_time)
        delay_communication_execution = False

        for h in context.hosts:
            # Omit finished hosts
            if h.finished():
                continue
            # Omit current host
            if h == context.get_current_host():
                continue
            # Omit hosts not using current channel
            if not channel.is_connected_with_host(h):
                continue

            # Check the time of other hosts
            host_time = self.module.get_current_time(self.simulator, h)

            # If hosts have the same time - the IN instruction should be executed before OUT execution
            # and other instructions (non-communication) should be executed before as well
            if host_time == min_hosts_time[1]:
                for instructions_context in h.get_all_instructions_contexts():
                    if instructions_context.finished():
                        continue
                    host_current_instruction = instructions_context.get_current_instruction()
                    if isinstance(host_current_instruction, CommunicationInstruction):
                        if instruction.is_out() and not host_current_instruction.is_out():
                            host_channel = context.channels_manager.find_channel_for_host_instruction(
                                context, h, host_current_instruction)
                            if host_channel:
                                if not host_channel.is_waiting_on_instruction(h, host_current_instruction):
                                    delay_communication_execution = True
                    else:
                        delay_communication_execution = True

            # If checked (in loop) host is in the past relative to the time of actual host
            if host_time < min_hosts_time[1]:

                # In general, the instruction in host with smaller time
                # should be executed earlier
                host_delay_communication_execution = True

                # The only exception is when the host in the past is waiting on IN instruction
                # and the request cannot be fulfilled
                current_instruction = h.get_current_instructions_context().get_current_instruction()
                if isinstance(current_instruction, CommunicationInstruction):
                    if not current_instruction.is_out():
                        # The checked host (the one in the past) is waiting for the message
                        # Lets check if the messages for the host are in the channel
                        host_channel = context.channels_manager.find_channel_for_host_instruction(
                            context, h, current_instruction)
                        if host_channel:

                            # If host is not waiting on IN instruction - let it go first
                            # If host is waiting - check if it is fulfilled
                            if host_channel.is_waiting_on_instruction(h, current_instruction):
                                request = host_channel.get_existing_request_for_instruction(h, current_instruction)
                                if not request.ready_to_fulfill():
                                    host_delay_communication_execution = False

                if host_delay_communication_execution:
                    delay_communication_execution = True

        ## Delay execution of this instruction
        ## if needed according to previous check
        if delay_communication_execution:
            return ExecutionResult(custom_index_management=True,
                                   finish_instruction_execution=True,
                                   result_kwargs=kwargs)

        ##############################################
        # Now the host with minimal time is executed
        ##############################################

        if instruction.is_out():
            # OUT instruction
            sender = context.get_current_host()
            # Get list of messages being sent from upper executor or build new list
            if 'sent_message' in kwargs:
                sent_message = kwargs['sent_message']
            else:
                sent_message = context.channels_manager.build_message(
                    context.get_current_host(),
                    context.get_current_host().get_variable(instruction.variable_name).clone(),
                    context.expression_checker)
                kwargs['sent_message'] = sent_message

            sender_time = self.module.get_current_time(self.simulator, sender)
            # DEBUG #
            # print 'sending message from ', sender.name, 'at', sender_time
            # DEBUG #

            all_requests = channel.get_filtered_requests(sent_message, context.channels_manager.get_router())
            accepted_requests = []
            for request in all_requests:
                if self.module.get_request_created_time(self.simulator, request) > sender_time:
                    sent_message.cancel_for_request(request)
                else:
                    accepted_requests.append(request)

            if len(accepted_requests) > 1:
                # broadcast
                # Send message with broadcast metric, update sender's time and fill in the sending time in message
                broadcast_time = self._get_time_of_sending_boradcast(context, channel, sent_message)
                self.module.add_message_sent_time(self.simulator, sent_message, sender_time)
                self.module.add_message_sending_time(self.simulator, sent_message, broadcast_time)

                self.module.set_current_time(self.simulator, sender, sender_time + broadcast_time)

                for request in accepted_requests:
                    if request.is_waiting and request.assigned_message is None:
                        receiving_time = self._get_time_of_receiving(context, channel, sent_message, request)
                        receiving_time = max(receiving_time, broadcast_time)

                        started_waiting_at = self.module.get_request_created_time(self.simulator, request)
                        receiver_time = self.module.get_current_time(self.simulator, request.receiver)
                        receiver_time = max(receiver_time, sender_time)
                        self.module.set_current_time(self.simulator, request.receiver,
                                                     receiver_time + receiving_time)
                        # Add timetrace to module
                        self.module.add_channel_message_trace(self.simulator, channel, sent_message,
                                                              sent_message.sender, sender_time, broadcast_time,
                                                              request.receiver, started_waiting_at,
                                                              receiver_time, receiving_time)

                # DEBUG #
                # print 'msg from', sent_message.sender.name, \
                #     'at', sender_time, '(', broadcast_time, 'ms ) ', 'to', len(accepted_requests), 'of', \
                #     len(all_requests), 'hosts', '- message', unicode(sent_message.expression)
                # DEBUG #

            elif len(accepted_requests) == 1:
                # point to point
                # Send message with link metric, update sender's time and fill in the sending time in message
                request = accepted_requests[0]
                sending_time = self._get_time_of_sending_point_to_point(context, channel,
                                                                        sent_message, request)
                self.module.add_message_sent_time(self.simulator, sent_message, sender_time)
                self.module.add_message_sending_time(self.simulator, sent_message, sending_time)

                self.module.set_current_time(self.simulator, sender, sender_time + sending_time)

                receiving_time = self._get_time_of_receiving(context, channel, sent_message, request)
                receiving_time = max(receiving_time, sending_time)

                receiver_time = self.module.get_current_time(self.simulator, request.receiver)
                receiver_time = max(receiver_time, sender_time)
                self.module.set_current_time(self.simulator, request.receiver,
                                             receiver_time + receiving_time)

                started_waiting_at = self.module.get_request_created_time(self.simulator, request)
                # Add timetrace to module
                self.module.add_channel_message_trace(self.simulator, channel, sent_message,
                                                      sent_message.sender, sender_time, sending_time,
                                                      request.receiver, started_waiting_at,
                                                      receiver_time, receiving_time)
                # DEBUG #
                # print 'msg from', sent_message.sender.name, 'to', request.receiver.name, \
                #     'at', sender_time, '(', sending_time, 'ms ) ', 'and started receiving at', \
                #     receiver_time, '(t:', receiving_time, 'ms, wait since:', started_waiting_at, 'ms)', \
                #     'message', unicode(sent_message.expression)
                # DEBUG #

            else:  # zero receivers
                # Send message with broadcast metric and update sender's time - no receivers
                broadcast_time = self._get_time_of_sending_boradcast(context, channel, sent_message)
                self.module.add_message_sent_time(self.simulator, sent_message, sender_time)
                self.module.add_message_sending_time(self.simulator, sent_message, broadcast_time)

                self.module.set_current_time(self.simulator, sender, sender_time + broadcast_time)

                # DEBUG #
                # print 'all requests', len(all_requests), ' - accepted:', len(accepted_requests)
                # print 'msg from', sent_message.sender.name, \
                #     'at', sender_time, '(', broadcast_time, 'ms ) ', \
                #     'message', unicode(sent_message.expression)
                # DEBUG #

        else:
            # IN instruction
            if 'messages_request' in kwargs:
                request = kwargs['messages_request']
            else:
                request = context.channels_manager.build_message_request(context.get_current_host(),
                                                                         instruction,
                                                                         context.expression_populator)
                kwargs['messages_request'] = request

            # If waiting request has NOT been created and added before
            if not channel.is_waiting_on_instruction(request.receiver, request.instruction):
                receiver = context.get_current_host()
                self.module.add_request_created_time(self.simulator, request,
                                                     self.module.get_current_time(self.simulator, receiver))
                # DEBUG #
                # print 'msg requested by', receiver.name, 'at', self.module.get_current_time(self.simulator, receiver)
                # DEBUG #


            if request.assigned_message is None:
                # Set messages from the past as cancelled for this request
                all_messages = channel.get_filtered_messages(request, context.channels_manager.get_router())
                accepted_messages = []
                for message in all_messages:
                    if self.module.get_request_created_time(self.simulator, request) > \
                            self.module.get_message_sent_time(self.simulator, message):
                        message.cancel_for_request(request)
                    else:
                        accepted_messages.append(message)

                if len(accepted_messages) > 0:
                    message = accepted_messages[0]
                    sending_time = self.module.get_message_sending_time(self.simulator, message)
                    receiving_time = self._get_time_of_receiving(context, channel, message, request)
                    receiving_time = max(receiving_time, sending_time)

                    receiver_time = self.module.get_current_time(self.simulator, request.receiver)
                    self.module.set_current_time(self.simulator, request.receiver,
                                                 receiver_time + receiving_time)

                    # Add timetrace to module
                    message_sent_time = self.module.get_message_sent_time(self.simulator, message)
                    started_waiting_at = self.module.get_request_created_time(self.simulator, request)
                    self.module.add_channel_message_trace(self.simulator, channel, message,
                                                          message.sender, message_sent_time, sending_time,
                                                          request.receiver, started_waiting_at,
                                                          receiver_time, receiving_time)
        return ExecutionResult(result_kwargs=kwargs)
        


#!/usr/bin/env python

import wx
import os

"""
@file       general_purpose_frame_gui.py
@brief      GUI (frame = window) for modules configuration, results presentation, etc...
@author     Katarzyna Mazur
@date       created on 19-06-2014 by Katarzyna Mazur
"""

class GeneralFrame(wx.Frame):
    """
    @brief frame (simply: window) for configuring chosen modules
    """

    def __init__(self, parent, windowTitle, boxTitle, icon):
        wx.Frame.__init__(self, parent)

        self.SetIcon(wx.Icon(self.CreatePath4Resource(icon), wx.BITMAP_TYPE_PNG))
        self.SetTitle(windowTitle)

        # create static box aka group box
        self.groupBox = wx.StaticBox(self, label=boxTitle)
        self.groupBoxSizer = wx.StaticBoxSizer(self.groupBox, wx.VERTICAL)

        # create buttons - simple 'OK' and 'Cancel' will be enough
        self.OKButton = wx.Button(self, label="OK")
        self.cancelButton = wx.Button(self, label="Cancel")

        # do some bindings - for now on OK, as well as the cancel button
        # will simply close the module configuration window
        self.OKButton.Bind(wx.EVT_BUTTON, self.OnOKButtonClicked)
        self.cancelButton.Bind(wx.EVT_BUTTON, self.OnCancelButtonClicked)

        # align buttons
        bottomSizer = wx.BoxSizer(wx.HORIZONTAL)
        bottomSizer.Add(wx.StaticText(self), 1, wx.EXPAND, 5)
        bottomSizer.Add(self.OKButton, 0, wx.ALIGN_CENTER | wx.ALL, 5)
        bottomSizer.Add(self.cancelButton, 0, wx.ALIGN_CENTER | wx.ALL, 5)

        self.sizer = wx.BoxSizer(wx.VERTICAL)
        self.sizer.Add(self.groupBoxSizer, 1, wx.EXPAND | wx.ALL, 5)
        self.sizer.Add(bottomSizer, 0, wx.EXPAND, 5)
        self.SetSizer(self.sizer)
        self.CentreOnScreen()
        self.Layout()

    def AddPanel(self, panel) :
        self.panel = panel
        self.groupBoxSizer.Add(self.panel, 1, wx.EXPAND | wx.ALL, 5)
        self.SetSizer(self.sizer)
        self.sizer.Layout()

    def DoFit(self):
        self.sizer.Fit(self)
        self.CentreOnScreen()
        self.Layout()

    def SetWindowSize(self, width, height):
        self.SetClientSize(wx.Size(width, height))
        self.CentreOnScreen()
        self.Layout()

    def OnOKButtonClicked(self, event) :
        self.Close()

    def OnCancelButtonClicked(self, event) :
        self.Close()

    def CreatePath4Resource(self, resourceName):
        """
        @brief      creates and returns path to the
                    given file in the resource
                    ('assets') dir
        @return     path to the resource
        """
        tmp = os.path.split(os.path.dirname(__file__))
        return os.path.join(tmp[0], 'bin', 'assets', resourceName)


#!/usr/bin/env python

import wx
import wx.combo

"""
@file       combo_check_box.py
@brief      mixed combo-check-box widget, needed for modules configuration
            and versions choosing [and probably some more GUI stuff]
@author     Katarzyna Mazur
@date       created on 12-05-2014 by Katarzyna Mazur
"""

class ComboCheckBox(wx.combo.ComboPopup):

    def Init(self):
        """
        @brief overrides Init method from the ComboPopup base class
        """

        # possible choices (checkboxes) displayed in combobox, empty at first
        self.choicesList = []

    def OnComboKeyEvent(self, event):
        """
        @brief receives key events from the parent ComboCtrl
        """
        wx.ComboPopup.OnComboKeyEvent(self, event)

    def Create(self, parent):
        """
        @brief creates the popup child control, returns true for success
        """

        # create checkbox list - use default wxpython's widget
        self.checkBoxList = wx.CheckListBox(parent)
        # react on checking / unchecking checkboxes in the checkbox list
        self.checkBoxList.Bind(wx.EVT_LISTBOX, self.OnListBoxClicked)
        return True

    def ClearChoices(self):
        del self.choicesList[:]

    def SetChoices(self, choices):
        """
        @brief initializes combobox with checkboxes values
        """

        # clear current content from combobox
        self.checkBoxList.Clear()
        # add all the elements from the list to the combo -
        self.checkBoxList.AppendItems(choices)

    def CheckIfAnythingIsSelected(self):
        """
        @brief checks if anything is selected on combobox,
        if at least one item is selected, returns True,
        otherwise False
        """
        for i in range(self.checkBoxList.GetCount()) :
            if self.checkBoxList.IsChecked(i) :
                return True
        return False

    def GetSelectedItemsCount(self):
        """
        @brief counts selected items (checkboxes) in
        our combobox, returns the number of the
        checked checkboxes
        """
        count = 0
        for i in range(self.checkBoxList.GetCount()) :
            if self.checkBoxList.IsChecked(i) :
                count += 1
        return count

    def GetControl(self):
        """
        @brief returns the widget that is to be used for the popup
        """
        return self.checkBoxList

    def OnListBoxClicked(self, evt):
        """
        @brief returns selected items in checkbox list
        """
        return [str(self.checkBoxList.GetString(i)) for i in range(self.checkBoxList.GetCount()) if
                        self.checkBoxList.IsChecked(i)]

    def GetSelectedItems(self):
        """
        @brief returns selected items in checkbox list
        """
        return [str(self.checkBoxList.GetString(i)) for i in range(self.checkBoxList.GetCount()) if
                        self.checkBoxList.IsChecked(i)]

    def OnPopup(self):
        """
        @brief called immediately after the popup is shown
        """
        wx.combo.ComboPopup.OnPopup(self)

    def GetAdjustedSize(self, minWidth, prefHeight, maxHeight):
        """
        @brief returns final size of popup - set prefHeight to 100
        in order to drop-down the popup and add vertical scrollbar
        if needed
        """
        return wx.combo.ComboPopup.GetAdjustedSize(self, minWidth, 100, maxHeight)

"""
@brief  Simple testing (example usage), should be removed or commented
"""

"""
# fast'n'dirty code for testing purposes ONLY!
class TestMePanel(wx.Panel):
    def __init__(self, parent, log):
        self.log = log
        wx.Panel.__init__(self, parent, -1)
        fgs = wx.FlexGridSizer(cols=3, hgap=10, vgap=10)

        # this is how we should use this combo check box widget
        # EXAMPLE USAGE
        cc = wx.combo.ComboCtrl(self)
        self.tcp = ComboCheckBox()
        cc.SetPopupControl(self.tcp)
        self.tcp.SetChoices(['Time Analysis', 'Energy consumption'])
        # second 'Set' removes previous - that's perfectly what we want
        self.tcp.SetChoices(['Time Analysis', 'Energy consumption', 'Reputation'])
        cc.SetText('...')

        # other, non important stuff
        fgs.Add(cc, 1, wx.EXPAND | wx.ALL, 20)
        butt = wx.Button(self, -0, "Check Selected")
        fgs.Add(butt, 1, wx.EXPAND | wx.ALL, 20)
        butt.Bind(wx.EVT_BUTTON, self.OnButtonClicked)
        box = wx.BoxSizer()
        box.Add(fgs, 1, wx.EXPAND | wx.ALL, 20)
        self.CentreOnParent()
        self.SetSizer(box)

    def OnButtonClicked(self, event=None):
        print self.tcp.GetSelectedItems()


class MainFrame(wx.Frame):
    def __init__(self, parent):
        wx.Frame.__init__(self, parent)
        panel = TestMePanel(self, None)


if __name__ == "__main__":
    app = wx.App(0)
    frame = MainFrame(None)
    frame.CenterOnScreen()
    frame.Show()
    app.MainLoop()
"""


#!/usr/bin/env python

import sys
from aqopa.simulator.state import Hook

"""
@file       console.py
@author     Katarzyna Mazur
"""

class PrintResultsHook(Hook):

    def __init__(self, module, simulator, output_file=sys.stdout):
        """ """
        self.module = module
        self.simulator = simulator
        self.output_file = output_file

    def execute(self, context, **kwargs):
        """ """

        self.output_file.write('-'*80)
        self.output_file.write('\n')
        self.output_file.write('Module\tFinance Analysis (cost in $)')
        self.output_file.write('\n')
        self.output_file.write('Version\t%s\n\n' % self.simulator.context.version.name)

        # default cost per one kilowatt-hour
        cost_per_kWh = 0.15

        costs = self.module.calculate_all_costs(self.simulator, context.hosts, cost_per_kWh)

        mincost, minhost = self.module.get_min_cost(self.simulator, context.hosts)
        maxcost, maxhost = self.module.get_max_cost(self.simulator, context.hosts)
        totalcost = self.module.get_total_cost(self.simulator, context.hosts)
        avgcost = self.module.get_avg_cost(self.simulator, context.hosts)

        self.output_file.write('Minimal cost:\t\t{0}\tHost: {1:}\t\n'.format(str(mincost), minhost.name))
        self.output_file.write('Maximal cost: \t\t{0}\tHost: {1:}\t\n'.format(str(maxcost), maxhost.name))
        self.output_file.write('Total cost:\t\t{0}\n'.format(str(totalcost)))
        self.output_file.write('Average cost:\t\t{0}\n'.format(str(avgcost)))

        self.output_file.write("\nActual costs:\n")
        for host in context.hosts:
            self.output_file.write('{0}\t\t{1:}\t'.format(host.name, str(costs[host])))
            self.output_file.write("\n")